@@ -1,7 +1,7 @@
 # -*- coding: utf-8 -*-
 # vim: expandtab ts=4 sw=4 sts=4:
 #
-# Copyright (C) 2007-2008 GNS3 Dev Team
+# Copyright (C) 2007-2010 GNS3 Development Team (http://www.gns3.net/team).
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License version 2 as
@@ -16,18 +16,14 @@
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 #
-# Contact: contact@gns3.net
+# code@gns3.net
 #
 
-import os, sys, time
+import os
 import GNS3.Globals as globals
-import GNS3.Config.Defaults as Defaults
-from GNS3.Config.Objects import iosImageConf, hypervisorConf, libraryConf
+from GNS3.Config.Objects import iosImageConf, hypervisorConf, libraryConf, qemuImageConf, pixImageConf, junosImageConf, asaImageConf, idsImageConf
 from GNS3.Globals.Symbols import SYMBOLS, SYMBOL_TYPES
 from GNS3.Node.DecorativeNode import DecorativeNode
-from GNS3.Node.IOSRouter import IOSRouter
-from GNS3.Link.Serial import Serial
-from GNS3.Link.Ethernet import Ethernet
 from PyQt4 import QtCore
 from GNS3.Utils import Singleton, translate
 
@@ -64,7 +60,7 @@ class ConfDB(Singleton, QtCore.QSettings):
             if default_value is not None:
                 return default_value
             # or return the app default if it exist
-            if _ConfigDefaults.has_key(key):
+            if ConfigDefaults.has_key(key):
                 return _ConfigDefaults[key]
             # or finally, return None
             return None
@@ -124,7 +120,9 @@ class ConfDB(Singleton, QtCore.QSettings):
             # Do the convertion, and assign the value
             if   conv_to == "string": __values[child] = str(child_value.toString())
             elif conv_to == "int":    __values[child] = int(child_value.toInt())
-            else: raise "convertion type not implemented"
+            else:
+                # convertion type not implemented
+                pass
 
         self.endGroup()
         return __values
@@ -180,6 +178,7 @@ class GNS_Conf(object):
             conf = iosImageConf()
             conf.id = int(img_num)
             conf.filename = img_filename
+            conf.baseconfig = unicode(c.get(cgroup + "/baseconfig", 'baseconfig.txt'))
             conf.platform = str(c.get(cgroup + "/platform", ''))
             conf.chassis = str(c.get(cgroup + "/chassis", ''))
             conf.idlepc = str(c.get(cgroup + "/idlepc", ''))
@@ -211,6 +210,7 @@ class GNS_Conf(object):
             hyp_wdir = c.get(cgroup + "/working_directory", unicode(''))
             hyp_baseUDP = c.get(cgroup + "/base_udp", '10000')
             hyp_baseConsole = c.get(cgroup + "/base_console", '2000')
+            hyp_baseAUX = c.get(cgroup + "/base_aux", '2500')
 
             # We need at least `hyp_host' and `hyp_port' to be set
             if hyp_host == '' or hyp_port == '':
@@ -225,11 +225,166 @@ class GNS_Conf(object):
             conf.workdir = hyp_wdir
             conf.baseUDP = int(hyp_baseUDP)
             conf.baseConsole = int(hyp_baseConsole)
+            conf.baseAUX = int(hyp_baseAUX)
             globals.GApp.hypervisors[img_ref] = conf
 
             if conf.id >= globals.GApp.hypervisors_ids:
                 globals.GApp.hypervisors_ids = conf.id + 1
 
+    def QEMU_images(self):
+        """ Load Qemu images settings from config file
+        """
+
+        # Loading Qemu image conf
+        basegroup = "QEMU.images"
+        c = ConfDB()
+        c.beginGroup(basegroup)
+        childGroups = c.childGroups()
+        c.endGroup()
+
+        for id in childGroups:
+
+            cgroup = basegroup + '/' + id
+            
+            conf = qemuImageConf()
+            conf.id = int(id)
+            conf.name = c.get(cgroup + "/name", unicode(''))
+            conf.filename = c.get(cgroup + "/filename", unicode(''))
+            conf.memory = int(c.get(cgroup + "/memory", 256))
+            conf.nic_nb = int(c.get(cgroup + "/nic_nb", 6))
+            conf.nic = str(c.get(cgroup + "/nic", 'e1000'))
+            conf.options = str(c.get(cgroup + "/options", ''))
+            conf.kqemu = c.value(cgroup + "/kqemu", QtCore.QVariant(False)).toBool()
+            conf.kvm = c.value(cgroup + "/kvm", QtCore.QVariant(False)).toBool()
+            globals.GApp.qemuimages[conf.name] = conf
+            
+            if conf.id >= globals.GApp.qemuimages_ids:
+                globals.GApp.qemuimages_ids = conf.id + 1
+                
+    def PIX_images(self):
+        """ Load PIX images settings from config file
+        """
+
+        # Loading PIX image conf
+        basegroup = "PIX.images"
+        c = ConfDB()
+        c.beginGroup(basegroup)
+        childGroups = c.childGroups()
+        c.endGroup()
+
+        for id in childGroups:
+
+            cgroup = basegroup + '/' + id
+            
+            conf = pixImageConf()
+            conf.id = int(id)
+            conf.name = c.get(cgroup + "/name", unicode(''))
+            conf.filename = c.get(cgroup + "/filename", unicode(''))
+            conf.memory = int(c.get(cgroup + "/memory", 128))
+            conf.nic_nb = int(c.get(cgroup + "/nic_nb", 6))
+            conf.nic = str(c.get(cgroup + "/nic", 'e1000'))
+            conf.options = str(c.get(cgroup + "/options", ''))
+            conf.kqemu = c.value(cgroup + "/kqemu", QtCore.QVariant(False)).toBool()
+            conf.key = str(c.get(cgroup + "/key", ''))
+            conf.serial = str(c.get(cgroup + "/serial", ''))
+            globals.GApp.piximages[conf.name] = conf
+            
+            if conf.id >= globals.GApp.piximages_ids:
+                globals.GApp.piximages_ids = conf.id + 1
+                
+    def JUNOS_images(self):
+        """ Load JunOS images settings from config file
+        """
+
+        # Loading JunOS image conf
+        basegroup = "JUNOS.images"
+        c = ConfDB()
+        c.beginGroup(basegroup)
+        childGroups = c.childGroups()
+        c.endGroup()
+
+        for id in childGroups:
+
+            cgroup = basegroup + '/' + id
+            
+            conf = junosImageConf()
+            conf.id = int(id)
+            conf.name = c.get(cgroup + "/name", unicode(''))
+            conf.filename = c.get(cgroup + "/filename", unicode(''))
+            conf.memory = int(c.get(cgroup + "/memory", 128))
+            conf.nic_nb = int(c.get(cgroup + "/nic_nb", 6))
+            conf.nic = str(c.get(cgroup + "/nic", 'e1000'))
+            conf.options = str(c.get(cgroup + "/options", ''))
+            conf.kqemu = c.value(cgroup + "/kqemu", QtCore.QVariant(False)).toBool()
+            conf.kvm = c.value(cgroup + "/kvm", QtCore.QVariant(False)).toBool()
+            globals.GApp.junosimages[conf.name] = conf
+
+            if conf.id >= globals.GApp.junosimages_ids:
+                globals.GApp.junosimages_ids = conf.id + 1
+                
+    def ASA_images(self):
+        """ Load ASA images settings from config file
+        """
+
+        # Loading ASA image conf
+        basegroup = "ASA.images"
+        c = ConfDB()
+        c.beginGroup(basegroup)
+        childGroups = c.childGroups()
+        c.endGroup()
+
+        for id in childGroups:
+
+            cgroup = basegroup + '/' + id
+            
+            conf = asaImageConf()
+            conf.id = int(id)
+            conf.name = c.get(cgroup + "/name", unicode(''))
+            conf.memory = int(c.get(cgroup + "/memory", 256))
+            conf.nic_nb = int(c.get(cgroup + "/nic_nb", 6))
+            conf.nic = str(c.get(cgroup + "/nic", 'e1000'))
+            conf.options = str(c.get(cgroup + "/options", ''))
+            conf.kqemu = c.value(cgroup + "/kqemu", QtCore.QVariant(False)).toBool()
+            conf.kvm = c.value(cgroup + "/kvm", QtCore.QVariant(False)).toBool()
+            conf.initrd = c.get(cgroup + "/initrd", unicode(''))
+            conf.kernel = c.get(cgroup + "/kernel", unicode(''))
+            conf.kernel_cmdline = c.get(cgroup + "/kernel_cmdline", unicode(''))
+            globals.GApp.asaimages[conf.name] = conf
+
+            if conf.id >= globals.GApp.asaimages_ids:
+                globals.GApp.asaimages_ids = conf.id + 1
+                
+    def IDS_images(self):
+        """ Load IDS images settings from config file
+        """
+
+        # Loading IDS image conf
+        basegroup = "IDS.images"
+        c = ConfDB()
+        c.beginGroup(basegroup)
+        childGroups = c.childGroups()
+        c.endGroup()
+
+        for id in childGroups:
+
+            cgroup = basegroup + '/' + id
+            
+            conf = idsImageConf()
+            conf.id = int(id)
+            conf.image1 = c.get(cgroup + "/image1", unicode(''))
+            conf.image2 = c.get(cgroup + "/image2", unicode(''))
+            conf.name = c.get(cgroup + "/name", unicode(''))
+            conf.memory = int(c.get(cgroup + "/memory", 512))
+            conf.nic_nb = int(c.get(cgroup + "/nic_nb", 3))
+            conf.nic = str(c.get(cgroup + "/nic", 'e1000'))
+            conf.options = str(c.get(cgroup + "/options", ''))
+            conf.kqemu = c.value(cgroup + "/kqemu", QtCore.QVariant(False)).toBool()
+            conf.kvm = c.value(cgroup + "/kvm", QtCore.QVariant(False)).toBool()
+            globals.GApp.idsimages[conf.name] = conf
+
+            if conf.id >= globals.GApp.idsimages_ids:
+                globals.GApp.idsimages_ids = conf.id + 1
+
     def Libraries(self):
         """ Load libraries settings from config file
         """
