@@ -1,7 +1,7 @@
 # -*- coding: utf-8 -*-
 # vim: expandtab ts=4 sw=4 sts=4:
 #
-# Copyright (C) 2007-2008 GNS3 Dev Team
+# Copyright (C) 2007-2010 GNS3 Development Team (http://www.gns3.net/team).
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License version 2 as
@@ -16,7 +16,7 @@
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 #
-# Contact: contact@gns3.net
+# code@gns3.net
 #
 
 SysConfigDir = "/etc/gns3"
@@ -33,6 +33,7 @@ conf_library_types = {
 conf_iosImage_defaults = {
     'id': -1,
     'filename': '',
+    'baseconfig': '',
     'platform': '',
     'chassis': '',
     'idlepc': '',
@@ -44,6 +45,7 @@ conf_iosImage_defaults = {
 conf_iosImage_types = {
     'id': int,
     'filename': unicode,
+    'baseconfig': unicode,
     'platform': str,
     'chassis': str,
     'idlepc': str,
@@ -58,7 +60,8 @@ conf_hypervisor_defaults = {
     'port': 7200,
     'workdir': '',
     'baseUDP': 10000,
-    'baseConsole': 2000, 
+    'baseConsole': 2000,
+    'baseAUX': 2100,
     'used_ram':0, 
 }
 
@@ -68,10 +71,139 @@ conf_hypervisor_types = {
     'port': int,
     'workdir': unicode,
     'baseUDP': int,
-    'baseConsole': int, 
+    'baseConsole': int,
+    'baseAUX': int,
     'used_ram': int, 
 }
 
+conf_qemuImage_defaults = {
+    'id': -1,
+    'name': '',
+    'filename': '',
+    'memory': 256,
+    'nib_nb': 6,
+    'nic': 'e1000',
+    'options': '',
+    'kqemu': False,
+    'kvm': False
+}
+
+conf_qemuImage_types = {
+    'id': int,
+    'name': unicode,
+    'filename': unicode,
+    'memory': int,
+    'nic_nb': int,
+    'nic': str,
+    'options': str,
+    'kqemu': bool,
+    'kvm': bool
+}
+
+conf_pixImage_defaults = {
+    'id': -1,
+    'name': '',
+    'filename': '',
+    'memory': 128,
+    'nib_nb': 6,
+    'nic': 'e1000',
+    'options': '',
+    'kqemu': False,
+    'key': '',
+    'serial': ''
+}
+
+conf_pixImage_types = {
+    'id': int,
+    'name': unicode,
+    'filename': unicode,
+    'memory': int,
+    'nic_nb': int,
+    'nic': str,
+    'options': str,
+    'kqemu': bool,
+    'key': str,
+    'serial': str
+}
+
+conf_junosImage_defaults = {
+    'id': -1,
+    'name': '',
+    'filename': '',
+    'memory': 96,
+    'nib_nb': 6,
+    'nic': 'e1000',
+    'options': '',
+    'kqemu': False,
+    'kvm': False
+}
+
+conf_junosImage_types = {
+    'id': int,
+    'name': unicode,
+    'filename': unicode,
+    'memory': int,
+    'nic_nb': int,
+    'nic': str,
+    'options': str,
+    'kqemu': bool,
+    'kvm': bool
+}
+
+conf_asaImage_defaults = {
+    'id': -1,
+    'name': '',
+    'memory': 256,
+    'nib_nb': 6,
+    'nic': 'e1000',
+    'options': '',
+    'kqemu': False,
+    'kvm': False,
+    'kernel': '',
+    'initrd': '',
+    'kernel_cmdline': ''
+}
+
+conf_asaImage_types = {
+    'id': int,
+    'name': unicode,
+    'memory': int,
+    'nic_nb': int,
+    'nic': str,
+    'options': str,
+    'kqemu': bool,
+    'kvm': bool,
+    'kernel': unicode,
+    'initrd': unicode,
+    'kernel_cmdline': unicode,
+}
+
+conf_idsImage_defaults = {
+    'id': -1,
+    'name': '',
+    'image1': '',
+    'image2': '',
+    'memory': 512,
+    'nib_nb': 3,
+    'nic': 'e1000',
+    'options': '',
+    'kqemu': False,
+    'kvm': False
+}
+
+conf_idsImage_types = {
+    'id': int,
+    'name': unicode,
+    'image1': unicode,
+    'image2': unicode,
+    'memory': int,
+    'nic_nb': int,
+    'nic': str,
+    'options': str,
+    'kqemu': bool,
+    'kvm': bool
+}
+
 conf_systemDynamips_defaults = {
     'path': '',
     'port': 7200,
@@ -79,7 +211,9 @@ conf_systemDynamips_defaults = {
     'clean_workdir': True, 
     'baseUDP': 10000,
     'baseConsole': 2000,
+    'baseAUX': 2100,
     'ghosting': True,
+    'jitsharing': False,
     'sparsemem': False,
     'mmap': True,
     'memory_limit': 512,
@@ -95,7 +229,9 @@ conf_systemDynamips_types = {
     'clean_workdir': bool, 
     'baseUDP': int,
     'baseConsole': int,
+    'baseAUX': int,
     'ghosting': bool,
+    'jitsharing': bool,
     'sparsemem': bool,
     'mmap': bool,
     'memory_limit': int,
@@ -106,7 +242,10 @@ conf_systemDynamips_types = {
 
 conf_systemGeneral_defaults = {
     'lang': 'en',
-    'project_startup': True, 
+    'project_startup': True,
+    'relative_paths': True,
+    'slow_start': 0,
+    'autosave': 60,
     'term_cmd': '',
     'use_shell': True,
     'project_path': '.',
@@ -119,7 +258,10 @@ conf_systemGeneral_defaults = {
 
 conf_systemGeneral_types = {
     'lang': unicode,
-    'project_startup': bool, 
+    'project_startup': bool,
+    'relative_paths': bool,
+    'slow_start': int,
+    'autosave': int,
     'use_shell': bool,
     'term_cmd': unicode,
     'project_path': unicode,
@@ -142,42 +284,30 @@ conf_systemCapture_types = {
     'auto_start': bool
 }
 
-conf_systemPemu_defaults = {
-    'pemuwrapper_path':'',
-    'pemuwrapper_workdir':'',
-    'external_host':'',
-    'enable_PemuManager': True,
-    'import_use_PemuManager': True,
-    'PemuManager_binding': u'localhost', 
-    'default_pix_image': '',
-    'default_pix_key':'',
-    'default_pix_serial':'',
-    'default_base_flash':'',
+conf_systemQemu_defaults = {
+    'qemuwrapper_path':'',
+    'qemuwrapper_workdir':'',
+    'qemu_path':'qemu',
+    'qemu_img_path':'qemu-img',
+    'external_hosts':[],
+    'enable_QemuManager': True,
+    'import_use_QemuManager': True,
+    'QemuManager_binding': u'localhost',
+    'qemuwrapper_port': 10525,
+    'qemuwrapper_baseUDP': 20000,
+    'qemuwrapper_baseConsole': 3000,
 }
 
-conf_systemPemu_types = {
-    'pemuwrapper_path': unicode,
-    'pemuwrapper_workdir': unicode,
-    'external_host': unicode,
-    'enable_PemuManager': bool,
-    'import_use_PemuManager': bool,
-    'PemuManager_binding': unicode,
-    'default_pix_image': unicode,
-    'default_pix_key': str,
-    'default_pix_serial': str,
-    'default_base_flash': unicode,
-}
-
-conf_systemSimhost_defaults = {
-    'path': '',
-    'workdir': '',
-    'basePort': 8000,
-    'baseUDP': 40000,
-}
-
-conf_systemSimhost_types = {
-    'path': unicode,
-    'workdir': unicode,
-    'basePort': int,
-    'baseUDP': int,
+conf_systemQemu_types = {
+    'qemuwrapper_path': unicode,
+    'qemuwrapper_workdir': unicode,
+    'qemu_path': unicode,
+    'qemu_img_path': unicode,
+    'external_hosts': list,
+    'enable_QemuManager': bool,
+    'import_use_QemuManager': bool,
+    'QemuManager_binding': unicode,
+    'qemuwrapper_port': int,
+    'qemuwrapper_baseUDP': int,
+    'qemuwrapper_baseConsole': int,
 }
