@@ -19,10 +19,10 @@
 # code@gns3.net
 #
 
-import sys, os
+import sys, os, re
 import GNS3.Globals as globals
 from GNS3.Dynagen.dynagen import Dynagen, DEVICETUPLE
-from GNS3.Utils import translate, debug
+from GNS3.Utils import translate, debug, getWindowsInterfaces
 from PyQt4 import QtCore, QtGui
 
 class DynagenSub(Dynagen):
@@ -34,10 +34,68 @@ class DynagenSub(Dynagen):
         Dynagen.__init__(self)
         self.gns3_data = None
 
+    def check_replace_GUID_NIO(self, filename):
+        """ Check and replace non-existing GUID (network interface ID) on Windows
+        """
+        
+        file = open(filename,'r')
+        lines = file.readlines()
+        cregex = re.compile("^.*nio_gen_eth:(.*)")
+        niolist = []
+        
+        for currentline in lines:
+            currentline = currentline.lower().strip() 
+            match_obj = cregex.match(currentline)
+            if match_obj:
+                niolist.append(match_obj.group(1))
+                
+        niolist = set(niolist)
+        if len(niolist):
+            
+            rpcaps = getWindowsInterfaces()
+            interfaces = {}
+            for rpcap in rpcaps:
+                match = re.search(r"""^rpcap://(\\Device\\NPF_{[a-fA-F0-9\-]*}).*:(.*)""", rpcap)
+                interface_guid = str(match.group(1)).lower()
+                interfaces[interface_guid] = unicode(match.group(2)).strip()
+
+            for nio in niolist:
+                if not interfaces.has_key(nio): 
+                    (selection, ok) = QtGui.QInputDialog.getItem(globals.GApp.mainWindow, translate("DynagenSub", "NIO connection"),
+                                                        unicode(translate("DynagenSub", "%s cannot be found\nPlease choose an alternate network interface:")) % nio, interfaces.values(), 0, False)
+                    if ok:
+                        interface = ""
+                        for (key, name) in interfaces.iteritems():
+                            if name == unicode(selection):
+                                interface = key
+                                break
+
+                        index = 0
+                        for currentline in lines:
+                            match_obj = cregex.match(currentline.lower())
+                            if match_obj:
+                                currentline = currentline.replace(nio, interface)
+                                lines[index] = currentline
+                            index += 1      
+                    else:
+                        continue
+  
+        file.close()
+        
+        if len(niolist):
+            # write changes
+            file = open(filename,'w')
+            for line in lines:
+                file.write(line)
+            file.close()
+          
+
     def open_config(self, FILENAME):
         """ Open the config file
         """
 
+        if sys.platform.startswith('win'):
+            self.check_replace_GUID_NIO(FILENAME)
         config = Dynagen.open_config(self, FILENAME)
         self.filename = FILENAME
         self.gns3_data = None
@@ -47,13 +105,13 @@ class DynagenSub(Dynagen):
                 if os.path.exists(self.gns3_data['configs']):
                     projectConfigsDir = self.gns3_data['configs']
                 else:
-                    projectConfigsDir = os.path.join(os.path.dirname(FILENAME), self.gns3_data['configs'])
+                    projectConfigsDir = os.path.join(os.path.dirname(FILENAME), unicode(self.gns3_data['configs']))
                 globals.GApp.workspace.projectConfigs = os.path.abspath(projectConfigsDir)
             if self.gns3_data.has_key('workdir'):
                 if os.path.exists(self.gns3_data['workdir']):
                     projectWorkdir = self.gns3_data['workdir']
                 else:
-                    projectWorkdir = os.path.join(os.path.dirname(FILENAME), self.gns3_data['workdir'])
+                    projectWorkdir = os.path.join(os.path.dirname(FILENAME), unicode(self.gns3_data['workdir']))
                 globals.GApp.workspace.projectWorkdir = os.path.abspath(projectWorkdir)
             config.sections.remove('GNS3-DATA')
 
@@ -82,7 +140,7 @@ class DynagenSub(Dynagen):
 
                     # Check if this is a relative working directory path and convert to an absolute path if necessary
                     if server['workingdir']:
-                        abspath = os.path.join(os.path.dirname(FILENAME), server['workingdir'])
+                        abspath = os.path.join(os.path.dirname(FILENAME), unicode(server['workingdir']))
                         if os.path.exists(abspath):
                             server['workingdir'] = abspath
                             debug(unicode("Converting relative working directory path to absolute path: %s") % server['workingdir'])
@@ -95,8 +153,9 @@ class DynagenSub(Dynagen):
                         # ASA has no image
                         if device.name == '5520' and device['initrd'] and device['kernel']:
                             if not os.access(device['initrd'], os.F_OK):
-                                if globals.GApp.systconf['qemu'].default_asa_initrd:
-                                    initrd_name = globals.GApp.systconf['qemu'].default_asa_initrd
+                                
+                                if len(globals.GApp.asaimages.keys()):
+                                    initrd_name = globals.GApp.asaimages.values()[0].initrd
                                 else:
                                     QtGui.QMessageBox.critical(globals.GApp.mainWindow, 'DynagenSub',
                                         unicode(translate("ASA initrd", "ASA initrd %s cannot be found and cannot find an alternative initrd")) % device['initrd'])
@@ -106,8 +165,8 @@ class DynagenSub(Dynagen):
                                 device['initrd'] = initrd_name
                                 
                             if not os.access(device['kernel'], os.F_OK):
-                                if globals.GApp.systconf['qemu'].default_asa_kernel:
-                                    kernel_name = globals.GApp.systconf['qemu'].default_asa_kernel
+                                if len(globals.GApp.asaimages.keys()):
+                                    kernel_name = globals.GApp.asaimages.values()[0].kernel
                                 else:
                                     QtGui.QMessageBox.critical(globals.GApp.mainWindow, 'DynagenSub',
                                         unicode(translate("ASA kernel", "ASA kernel %s cannot be found and cannot find an alternative kernel")) % device['kernel'])
@@ -120,8 +179,9 @@ class DynagenSub(Dynagen):
                         # IDS has no default image
                         if device.name == 'IDS-4215' and device['image1'] and device['image2']:
                             if not os.access(device['image1'], os.F_OK):
-                                if globals.GApp.systconf['qemu'].default_ids_image1:
-                                    image1_name = globals.GApp.systconf['qemu'].default_ids_image1
+                                
+                                if len(globals.GApp.idsimages.keys()):
+                                    image1_name = globals.GApp.idsimages.values()[0].image1
                                 else:
                                     QtGui.QMessageBox.critical(globals.GApp.mainWindow, 'DynagenSub',
                                         unicode(translate("IDS image (hda)", "IDS image %s cannot be found and cannot find an alternative image")) % device['image1'])
@@ -131,8 +191,8 @@ class DynagenSub(Dynagen):
                                 device['image1'] = image1_name
                                 
                             if not os.access(device['image2'], os.F_OK):
-                                if globals.GApp.systconf['qemu'].default_ids_image2:
-                                    image2_name = globals.GApp.systconf['qemu'].default_ids_image2
+                                if len(globals.GApp.idsimages.keys()):
+                                    image2_name = globals.GApp.idsimages.values()[0].image2
                                 else:
                                     QtGui.QMessageBox.critical(globals.GApp.mainWindow, 'DynagenSub',
                                         unicode(translate("IDS image (hdb)", "IDS image %s cannot be found and cannot find an alternative image")) % device['image2'])
@@ -147,14 +207,14 @@ class DynagenSub(Dynagen):
                             continue
                         # Check if the image path is a relative path
                         if os.path.exists(device['image']) == False:
-                            abspath = os.path.join(os.path.dirname(FILENAME), device['image'])
+                            abspath = os.path.join(os.path.dirname(FILENAME), unicode(device['image']))
                             if os.path.exists(abspath):
                                 device['image'] = abspath
                         
                         if device.name == 'O-series' and device['image']:
                             if not os.access(device['image'], os.F_OK):
-                                if globals.GApp.systconf['qemu'].default_junos_image:
-                                    image_name = globals.GApp.systconf['qemu'].default_junos_image
+                                if len(globals.GApp.junosimages.keys()):
+                                    image_name = globals.GApp.junosimages.values()[0].filename
                                 else:
                                     QtGui.QMessageBox.critical(globals.GApp.mainWindow, 'DynagenSub',
                                         unicode(translate("JunOS image", "JunOS image %s cannot be found and cannot find an alternative image")) % device['image'])
@@ -162,6 +222,7 @@ class DynagenSub(Dynagen):
                                 print unicode(translate("DynagenSub", "Local JunOS image %s cannot be found, use image %s instead")) \
                                 % (unicode(device['image']), image_name)
                                 device['image'] = image_name
+
                         if device.name == 'QemuDevice' and device['image']:
                             if not os.access(device['image'], os.F_OK):
                                 if len(globals.GApp.qemuimages.keys()):
@@ -177,8 +238,8 @@ class DynagenSub(Dynagen):
                             # must be a PIX device
                             # check if the PIX image is accessible, if not find an alternative image
                             if not os.access(device['image'], os.F_OK):
-                                if globals.GApp.systconf['qemu'].default_pix_image:
-                                    image_name = globals.GApp.systconf['qemu'].default_pix_image
+                                if len(globals.GApp.piximages.keys()):
+                                    image_name = globals.GApp.piximages.values()[0].filename
                                 else:
                                     QtGui.QMessageBox.critical(globals.GApp.mainWindow, 'DynagenSub',
                                         unicode(translate("PIX image", "PIX image %s cannot be found and cannot find an alternative image")) % device['image'])
@@ -206,7 +267,7 @@ class DynagenSub(Dynagen):
                     
                     # Check if this is a relative working directory path and convert to an absolute path if necessary
                     if server['workingdir']:
-                        abspath = os.path.join(os.path.dirname(FILENAME), server['workingdir'])
+                        abspath = os.path.join(os.path.dirname(FILENAME), unicode(server['workingdir']))
                         if os.path.exists(abspath):
                             server['workingdir'] = abspath
                             debug(unicode("Converting relative working directory path to absolute path: %s") % server['workingdir'])
@@ -227,27 +288,36 @@ class DynagenSub(Dynagen):
 
                     for subsection in server.sections:
                         device = server[subsection]
+                        
                         # check if the IOS image is accessible, if not find an alternative image
                         if device.name in DEVICETUPLE:
-                            
                             # Check if this is a relative image path and convert to an absolute path if necessary
-                            abspath = os.path.join(os.path.dirname(FILENAME), device['image'])
+                            abspath = os.path.join(os.path.dirname(FILENAME), unicode(device['image']))
                             if os.path.exists(abspath):
                                 device['image'] = abspath
                                 debug(unicode("Converting relative image path to absolute path: %s") % device['image'])
 
                             if not os.access(device['image'], os.F_OK):
+                                
                                 selected_images = []
                                 image_to_use = None
                                 for (image, conf) in globals.GApp.iosimages.iteritems():
                                     if conf.chassis == device.name:
                                         selected_images.append(image)
-                                if len(selected_images) == 0:
+
+                                if len(selected_images):
+                                    message = "Local IOS image %s\ncannot be found for hypervisor %s\n\nPlease choose an alternative image:" % (unicode(device['image']), unicode(server.host) + ':' + controlPort)
+                                    (selection,  ok) = QtGui.QInputDialog.getItem(globals.GApp.mainWindow, translate("DynagenSub", "IOS image"),
+                                                                                      translate("DynagenSub", message), selected_images, 0, False)
+                                    if ok:
+                                        image_to_use = unicode(selection)
+
+                                if image_to_use == None and len(selected_images) == 0:
                                     QtGui.QMessageBox.critical(globals.GApp.mainWindow, 'DynagenSub', 
                                                                unicode(translate("IOS image", "IOS image %s cannot be found for hypervisor %s and cannot find an alternative %s image")) 
                                                                 % (device['image'], unicode(server.host) + ':' + controlPort, device.name))
                                     continue
-                                if len(selected_images) > 1:
+                                if image_to_use == None and len(selected_images) > 1:
                                     for image in selected_images:
                                         conf = globals.GApp.iosimages[image]
                                         if conf.default:
@@ -256,15 +326,19 @@ class DynagenSub(Dynagen):
                                 if not image_to_use:
                                     image_to_use = selected_images[0]
                                 image_name = globals.GApp.iosimages[image_to_use].filename
+                                ram = globals.GApp.iosimages[image_to_use].default_ram
+                                idlepc = globals.GApp.iosimages[image_to_use].idlepc
                                 print unicode(translate("DynagenSub", "Local IOS image %s cannot be found for hypervisor %s, use image %s instead")) \
                                 % (unicode(device['image']), unicode(server.host) + ':' + controlPort, image_name)
                                 device['image'] = image_name
+                                device['ram'] = ram
+                                device['idlepc'] = idlepc
 
                         # check if the config file is accessible, if not find an alternative config
                         elif device.has_key('cnfg') and device['cnfg']:
                             
                             # Check if this is a relative config path and convert to an absolute path if necessary
-                            abspath = os.path.join(os.path.dirname(FILENAME), device['cnfg'])
+                            abspath = os.path.join(os.path.dirname(FILENAME), unicode(device['cnfg']))
                             if os.path.exists(abspath):
                                 device['cnfg'] = abspath
                                 debug(unicode("Converting relative config path to absolute path: %s") % device['cnfg'])
@@ -306,5 +380,8 @@ class DynagenSub(Dynagen):
 
         print '\n*** Error:', unicode(msg)
         Dynagen.handled = True
-        self.doreset()
+        try:
+            self.doreset()
+        except:
+            print "Reset error, lost communication with hypervisor?"
         raise
