@@ -1,7 +1,7 @@
 # -*- coding: utf-8 -*-
 # vim: expandtab ts=4 sw=4 sts=4:
 #
-# Copyright (C) 2007-2008 GNS3 Dev Team
+# Copyright (C) 2007-2010 GNS3 Development Team (http://www.gns3.net/team).
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License version 2 as
@@ -16,13 +16,14 @@
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 #
-# Contact: contact@gns3.net
+# code@gns3.net
 #
 
 from PyQt4 import QtCore, QtGui
 from GNS3.Link.AbstractEdge import AbstractEdge
 from GNS3.Annotation import Annotation
 import GNS3.Globals as globals
+import math
 
 class Ethernet(AbstractEdge):
     """ Ethernet class
@@ -52,9 +53,9 @@ class Ethernet(AbstractEdge):
 
         # offset on the line for status points
         if self.length == 0:
-           self.edgeOffset = QtCore.QPointF(0, 0)
+            self.edgeOffset = QtCore.QPointF(0, 0)
         else:
-           self.edgeOffset = QtCore.QPointF((self.dx * 40) / self.length, (self.dy * 40) / self.length)
+            self.edgeOffset = QtCore.QPointF((self.dx * 40) / self.length, (self.dy * 40) / self.length)
 
     def shape(self):
         """ Return the shape of the item to the scene renderer
@@ -92,7 +93,7 @@ class Ethernet(AbstractEdge):
 
             # if nodes are too close, points disappears
             if self.length < 100:
-               return
+                return
 
             if self.src_interface_status == 'up':
                 color = QtCore.Qt.green
@@ -117,15 +118,31 @@ class Ethernet(AbstractEdge):
 
             if globals.GApp.workspace.flg_showInterfaceNames:
                 if self.labelSouceIf == None:
-                    self.labelSouceIf = Annotation()
-                    self.labelSouceIf.setPlainText(self.srcIf)
-                    self.labelSouceIf.setPos(point1)
-                    self.labelSouceIf.setZValue(2)
-                    globals.GApp.topology.addItem(self.labelSouceIf)
-                    self.labelSouceIf.autoGenerated = True
-                if not self.labelSouceIf.isVisible():
-                    self.labelSouceIf.setPos(point1)
+
+                    if globals.interfaceLabels.has_key(self.source.hostname + ' ' + self.srcIf):
+                        self.labelSouceIf = Annotation(self.source)
+                        annotation = globals.interfaceLabels[self.source.hostname + ' ' + self.srcIf]
+                        self.labelSouceIf.setZValue(annotation.zValue())
+                        self.labelSouceIf.setDefaultTextColor(annotation.defaultTextColor())
+                        self.labelSouceIf.setFont(annotation.font())
+                        self.labelSouceIf.setPlainText(annotation.toPlainText())
+                        self.labelSouceIf.setPos(annotation.x(), annotation.y())
+                        self.labelSouceIf.rotation = annotation.rotation
+                        self.labelSouceIf.rotate(annotation.rotation)
+                        del globals.interfaceLabels[self.source.hostname + ' ' + self.srcIf]
+                    elif not globals.GApp.workspace.flg_showOnlySavedInterfaceNames:
+                        self.labelSouceIf = Annotation(self.source)
+                        self.labelSouceIf.setPlainText(self.srcIf)
+                        self.labelSouceIf.setPos(self.mapToItem(self.source, point1))
+                        #self.labelSouceIf.autoGenerated = True
+
+                    if self.labelSouceIf:
+                        self.labelSouceIf.deviceName = self.source.hostname
+                        self.labelSouceIf.deviceIf = self.srcIf
+
+                if self.labelSouceIf and not self.labelSouceIf.isVisible():
                     self.labelSouceIf.show()
+
             elif self.labelSouceIf and globals.GApp.workspace.flg_showInterfaceNames == False:
                 self.labelSouceIf.hide()
 
@@ -154,15 +171,31 @@ class Ethernet(AbstractEdge):
 
             if globals.GApp.workspace.flg_showInterfaceNames:
                 if self.labelDestIf  == None:
-                    self.labelDestIf = Annotation()
-                    self.labelDestIf.setPlainText(self.destIf)
-                    self.labelDestIf.setPos(point2)
-                    self.labelDestIf.setZValue(2)
-                    globals.GApp.topology.addItem(self.labelDestIf)
-                    self.labelDestIf.autoGenerated = True
-                if not self.labelDestIf.isVisible():
-                    self.labelDestIf.setPos(point2)
+
+                    if globals.interfaceLabels.has_key(self.dest.hostname + ' ' + self.destIf):
+                        self.labelDestIf = Annotation(self.dest)
+                        annotation = globals.interfaceLabels[self.dest.hostname + ' ' + self.destIf]
+                        self.labelDestIf.setZValue(annotation.zValue())
+                        self.labelDestIf.setDefaultTextColor(annotation.defaultTextColor())
+                        self.labelDestIf.setFont(annotation.font())
+                        self.labelDestIf.setPlainText(annotation.toPlainText())
+                        self.labelDestIf.setPos(annotation.x(), annotation.y())
+                        self.labelDestIf.rotation = annotation.rotation
+                        self.labelDestIf.rotate(annotation.rotation)
+                        del globals.interfaceLabels[self.dest.hostname + ' ' + self.destIf]
+                    elif not globals.GApp.workspace.flg_showOnlySavedInterfaceNames:
+                        self.labelDestIf = Annotation(self.dest)
+                        self.labelDestIf.setPlainText(self.destIf)
+                        self.labelDestIf.setPos(self.mapToItem(self.dest, point2))
+                        #self.labelDestIf.autoGenerated = True
+
+                    if self.labelDestIf:
+                        self.labelDestIf.deviceName = self.dest.hostname
+                        self.labelDestIf.deviceIf = self.destIf
+
+                if self.labelDestIf and not self.labelDestIf.isVisible():
                     self.labelDestIf.show()
+
             elif self.labelDestIf and globals.GApp.workspace.flg_showInterfaceNames == False:
                 self.labelDestIf.hide()
 
