@@ -134,6 +134,7 @@ class FRSW(AbstractNode):
         self.config['ports'] = list(config['ports'])
         self.config['mapping'] = config['mapping'].copy()
         globals.GApp.topology.changed = True
+        self.mapping()
 
     def set_hypervisor(self,  hypervisor):
         """ Records a hypervisor
@@ -190,10 +191,10 @@ class FRSW(AbstractNode):
         self.create_config()
         return True
 
-    def startNode(self):
-        """ Start the node
+    def mapping(self):
+        """ Configure Frame-relay DLCI mapping
         """
-
+        
         connected_interfaces = map(int, self.getConnectedInterfaceList())
         for (source,  destination) in self.config['mapping'].iteritems():
             (srcport, srcdlci) = source.split(':')
@@ -202,8 +203,13 @@ class FRSW(AbstractNode):
                 debug('FRSW ' + self.hostname + ' is mapping: ' + source + ' to ' + destination)
                 if not self.frsw.pvcs.has_key((int(srcport), int(srcdlci))) and not self.frsw.pvcs.has_key((int(destport), int(destdlci))):
                     self.frsw.map(int(srcport), int(srcdlci), int(destport), int(destdlci))
-                    self.frsw.map(int(destport), int(destdlci), int(srcport), int(srcdlci))
+                    self.frsw.map(int(destport), int(destdlci), int(srcport), int(srcdlci))    
 
+    def startNode(self):
+        """ Start the node
+        """
+        
+        self.mapping()
         self.startupInterfaces()
         self.state = 'running'
         globals.GApp.mainWindow.treeWidget_TopologySummary.changeNodeStatus(self.hostname, 'running')
