@@ -19,7 +19,7 @@
 # code@gns3.net
 #
 
-import os, shutil, glob, sys
+import os, shutil, glob, sys, base64
 import GNS3.Globals as globals
 import GNS3.Dynagen.dynamips_lib as lib
 import GNS3.Dynagen.dynagen as dynagen_namespace
@@ -60,6 +60,8 @@ class IOSRouter(AbstractNode):
 
         # assign a new hostname
         global router_id
+        if not router_id:
+            router_id = 1
 
         # check if hostname has already been assigned
         for node in globals.GApp.topology.nodes.itervalues():
@@ -323,7 +325,7 @@ class IOSRouter(AbstractNode):
             if module and module.adapter != 'NM-16ESW':
                 interfaces = module.interfaces
                 if len(interfaces):
-                    type= interfaces.keys()[0]
+                    type = interfaces.keys()[0]
                     found = False
                     for port in interfaces[type].values():
                         if self.router.model_string in SLOTLESS_MODELS:
@@ -479,6 +481,12 @@ class IOSRouter(AbstractNode):
         """ Used when changing the hostname
         """
 
+        old_hostname = self.hostname
+        old_console = None
+        old_aux = None
+        if self.router:
+            old_console = self.router.console
+            old_aux = self.router.aux
         links = self.getEdgeList().copy()
         for link in links:
             globals.GApp.topology.deleteLink(link)
@@ -494,6 +502,10 @@ class IOSRouter(AbstractNode):
         self.set_hostname(new_hostname)
         try:
             self.create_router()
+            if old_console:
+                self.router.console = old_console
+            if old_aux:
+                self.router.aux = old_aux
         except lib.DynamipsError, msg:
             QtGui.QMessageBox.critical(globals.GApp.mainWindow, translate("IOSRouter", "Dynamips error"),  unicode(msg))
             self.delete_router()
@@ -502,6 +514,18 @@ class IOSRouter(AbstractNode):
         self.set_config(self.local_config)
         for link in links:
             globals.GApp.topology.addLink(link.source.id, link.srcIf, link.dest.id, link.destIf)
+            
+        # Update base config
+        try:
+            config = base64.decodestring(self.router.config_b64)
+            if config:
+                old_hostname
+                config = config.replace('\r', "")
+                config = '!\n' + config.replace(old_hostname, new_hostname)
+                encoded = ("").join(base64.encodestring(config).split())
+                self.router.config_b64 = encoded
+        except:
+            debug("Cannot change base config")
 
     def configNode(self):
         """ Node configuration
@@ -613,6 +637,15 @@ class IOSRouter(AbstractNode):
 
         if self.router and self.router.state == 'running' and self.router.console:
             console.connect(self.hypervisor.host,  self.router.console,  self.hostname)
+    
+    def isStarted(self):
+        """ Returns True if this router is started
+        """
+
+        if self.router and self.router.state == 'running':
+            return True
+        else:
+            return False
 
     def mousePressEvent(self, event):
         """ Call when the node is clicked
