@@ -38,3 +38,12 @@ class IOSRouter3700(IOSRouter, IOSRouter3700Defaults):
 
         assert(self.router)
         return ('')
+    
+    def create_router(self):
+
+        IOSRouter.create_router(self)
+        # Automatically assign NM-16ESW in slot 1
+        if not self.default_symbol:
+            config = self.create_config()
+            config['slots'][1] = 'NM-16ESW'
+            self.set_config(config)
