@@ -29,6 +29,7 @@ from GNS3.Annotation import Annotation
 from GNS3.ShapeItem import AbstractShapeItem, Rectangle, Ellipse
 from GNS3.StyleDialog import StyleDialog
 from GNS3.MACTableDialog import MACTableDialog
+from GNS3.SymbolDialog import SymbolDialog
 from GNS3.Pixmap import Pixmap
 from GNS3.NodeConfigurator import NodeConfigurator
 from GNS3.Node.AbstractNode import AbstractNode
@@ -65,6 +66,9 @@ class Scene(QtGui.QGraphicsView):
         self.newedge = None
         self.resetAddingLink()
         self.reloadRenderers()
+        
+        self.sceneDragging = False
+        self.lastMousePos = None 
 
     def reloadRenderers(self):
         """ Load all needed renderers
@@ -142,6 +146,15 @@ class Scene(QtGui.QGraphicsView):
             self.connect(MACTableAct, QtCore.SIGNAL('triggered()'), self.slotMACTable)
 
             menu.addAction(MACTableAct)
+            
+        instances = map(lambda item: isinstance(item, AbstractNode), items)
+        if True in instances:
+
+            # Action: Change symbol
+            changeSymbol = QtGui.QAction(translate('Scene', 'Change Symbol'), menu)
+            changeSymbol.setIcon(QtGui.QIcon(':/icons/node_conception.svg'))
+            self.connect(changeSymbol, QtCore.SIGNAL('triggered()'), self.slotchangeSymbol)        
+            menu.addAction(changeSymbol)            
 
         instances = map(lambda item: isinstance(item, IOSRouter) or isinstance(item, AnyEmuDevice), items)
         if True in instances:
@@ -150,7 +163,7 @@ class Scene(QtGui.QGraphicsView):
             consolePortAct = QtGui.QAction(translate('Scene', 'Change console port'), menu)
             consolePortAct.setIcon(QtGui.QIcon(':/icons/console_port.svg'))
             self.connect(consolePortAct, QtCore.SIGNAL('triggered()'), self.slotChangeConsolePort)
-
+            
             # Action: Console (Connect to the node console)
             consoleAct = QtGui.QAction(translate('Scene', 'Console'), menu)
             consoleAct.setIcon(QtGui.QIcon(':/icons/console.svg'))
@@ -165,15 +178,26 @@ class Scene(QtGui.QGraphicsView):
             stopAct = QtGui.QAction(translate('Scene', 'Stop'), menu)
             stopAct.setIcon(QtGui.QIcon(':/icons/stop.svg'))
             self.connect(stopAct, QtCore.SIGNAL('triggered()'), self.slotStopNode)
+            
+            # Action: Reload (stop and start IOS on hypervisor)
+            reloadAct = QtGui.QAction(translate('Scene', 'Reload'), menu)
+            reloadAct.setIcon(QtGui.QIcon(':/icons/reload.svg'))
+            self.connect(reloadAct, QtCore.SIGNAL('triggered()'), self.slotReloadNode)
 
             menu.addAction(consolePortAct)
             menu.addAction(consoleAct)
             menu.addAction(startAct)
             menu.addAction(stopAct)
+            menu.addAction(reloadAct)
 
         instances = map(lambda item: isinstance(item, IOSRouter), items)
         if True in instances:
 
+            # Action: Change the aux port
+            auxPortAct = QtGui.QAction(translate('Scene', 'Change aux port'), menu)
+            auxPortAct.setIcon(QtGui.QIcon(':/icons/console_port.svg'))
+            self.connect(auxPortAct, QtCore.SIGNAL('triggered()'), self.slotChangeAUXPort)
+
             # Action: Calculate IDLE PC
             idlepcAct = QtGui.QAction(translate('Scene', 'Idle PC'), menu)
             idlepcAct.setIcon(QtGui.QIcon(':/icons/calculate.svg'))
@@ -188,14 +212,9 @@ class Scene(QtGui.QGraphicsView):
             suspendAct = QtGui.QAction(translate('Scene', 'Suspend'), menu)
             suspendAct.setIcon(QtGui.QIcon(':/icons/pause.svg'))
             self.connect(suspendAct, QtCore.SIGNAL('triggered()'), self.slotSuspendNode)
-            
-            # Action: Reload (stop and start IOS on hypervisor)
-            reloadAct = QtGui.QAction(translate('Scene', 'Reload'), menu)
-            reloadAct.setIcon(QtGui.QIcon(':/icons/reload.svg'))
-            self.connect(reloadAct, QtCore.SIGNAL('triggered()'), self.slotReloadNode)
 
             menu.addAction(suspendAct)
-            menu.addAction(reloadAct)
+            menu.addAction(auxPortAct)
             menu.addAction(idlepcAct)
             menu.addAction(StartupConfigAct)
 
@@ -209,6 +228,13 @@ class Scene(QtGui.QGraphicsView):
 
             menu.addAction(styleAct)
             
+            # Action: Duplicate
+            duplicateAct = QtGui.QAction(translate('Scene', 'Duplicate'), menu)
+            duplicateAct.setIcon(QtGui.QIcon(':/icons/new.svg'))
+            self.connect(duplicateAct, QtCore.SIGNAL('triggered()'), self.slotDuplicate)
+
+            menu.addAction(duplicateAct)
+            
         # Action: Delete (Delete the node)
         deleteAct = QtGui.QAction(translate('Scene', 'Delete'), menu)
         deleteAct.setIcon(QtGui.QIcon(':/icons/delete.svg'))
@@ -224,10 +250,25 @@ class Scene(QtGui.QGraphicsView):
         raiseZvalueAct = QtGui.QAction(translate('Scene', 'Raise one step'), menu)
         raiseZvalueAct.setIcon(QtGui.QIcon(':/icons/raise_z_value.svg'))
         self.connect(raiseZvalueAct, QtCore.SIGNAL('triggered()'), self.slotraiseZValue)
-
+        
         menu.addAction(raiseZvalueAct)
         menu.addAction(lowerZvalueAct)
 
+        items = self.__topology.selectedItems()
+        if len(items) > 1:
+       
+            # Action: Align horizontally
+            hozAlignAct = QtGui.QAction(translate('Scene', 'Align horizontally'), menu)
+            hozAlignAct.setIcon(QtGui.QIcon(':/icons/horizontally.svg'))
+            self.connect(hozAlignAct, QtCore.SIGNAL('triggered()'), self.slotHozAlignment)
+            menu.addAction(hozAlignAct)
+            
+            # Action: Align vertically
+            vertAlignAct = QtGui.QAction(translate('Scene', 'Align vertically'), menu)
+            vertAlignAct.setIcon(QtGui.QIcon(':/icons/vertically.svg'))
+            self.connect(vertAlignAct, QtCore.SIGNAL('triggered()'), self.slotVertAlignment)
+            menu.addAction(vertAlignAct)
+
         menu.exec_(QtGui.QCursor.pos())
 
         # force the deletion of the children
@@ -261,6 +302,22 @@ class Scene(QtGui.QGraphicsView):
                 table.show()
                 table.exec_()
 
+    def slotHozAlignment(self):
+        """ Horizontally align items
+        """
+        
+        hozPos = self.__topology.selectedItems()[0].y()
+        for item in self.__topology.selectedItems():
+            item.setPos(item.x(), hozPos)
+            
+    def slotVertAlignment(self):
+        """ Vertically align items
+        """
+        
+        vertPos = self.__topology.selectedItems()[0].x()
+        for item in self.__topology.selectedItems():
+            item.setPos(vertPos, item.y())
+
     def slotStyle(self):
         """ Change the style of an annotation or a shape item
         """
@@ -284,7 +341,7 @@ class Scene(QtGui.QGraphicsView):
             for item in self.__topology.selectedItems():
                 if isinstance(item, Annotation):
                     command = undo.NewAnnotationStyle(item, style.color, style.font, style.rotation)
-                    item.autoGenerated = False
+                    #item.autoGenerated = False
                     self.__topology.undoStack.push(command)
                 elif isinstance(item, AbstractShapeItem):
                     pen = QtGui.QPen(style.borderColor, style.borderWidth, style.borderStyle, QtCore.Qt.RoundCap, QtCore.Qt.RoundJoin)
@@ -292,6 +349,40 @@ class Scene(QtGui.QGraphicsView):
                     command = undo.NewItemStyle(item, pen, brush, style.rotation)
                     self.__topology.undoStack.push(command)
 
+    def slotDuplicate(self):
+        """ Duplicate an item
+        """
+        
+        for item in self.__topology.selectedItems():
+            if isinstance(item, Annotation):
+                dupnote = Annotation()
+                dupnote.setPos(item.x(), item.y() - 20)
+                dupnote.setPlainText(item.toPlainText())
+                dupnote.setDefaultTextColor(item.defaultTextColor())
+                dupnote.setFont(item.font())
+                dupnote.rotate(item.rotation)
+                dupnote.rotation = item.rotation
+                dupnote.setZValue(item.zValue())
+                command = undo.AddItem(self.__topology, dupnote, translate("Scene", "annotation"))
+                self.__topology.undoStack.push(command)
+            if isinstance(item, AbstractShapeItem):
+                pos = QtCore.QPointF(item.x() - 20, item.y() - 20)
+                rect = item.rect()
+                if isinstance(item, Rectangle):
+                    dupshape = Rectangle(pos, QtCore.QSizeF(rect.width(), rect.height()))
+                else:
+                    dupshape = Ellipse(pos, QtCore.QSizeF(rect.width(), rect.height()))
+                dupshape.setPen(item.pen())
+                dupshape.setBrush(item.brush())
+                dupshape.rotate(item.rotation)
+                dupshape.rotation = item.rotation
+                dupshape.setZValue(item.zValue())
+                if isinstance(dupshape, Rectangle):
+                    command = undo.AddItem(self.__topology, dupshape, translate("Scene", "rectangle"))
+                else:
+                    command = undo.AddItem(self.__topology, dupshape, translate("Scene", "ellipse"))
+                self.__topology.undoStack.push(command)
+
     def slotIdlepc(self):
         """ Compute an IDLE PC
         """
@@ -341,7 +432,7 @@ class Scene(QtGui.QGraphicsView):
             if len(idles) == 0:
                 QtGui.QMessageBox.critical(globals.GApp.mainWindow, translate("Scene", "IDLE PC"),  translate("Scene", "No idlepc values found"))
                 return
-            (selection,  ok) = QtGui.QInputDialog.getItem(globals.GApp.mainWindow, translate("Scene", "IDLE PC"),
+            (selection, ok) = QtGui.QInputDialog.getItem(globals.GApp.mainWindow, translate("Scene", "IDLE PC"),
                                                         translate("Scene", "Potentially better idlepc values marked with '*'"), options, 0, False)
             if ok:
                 selection = str(selection).split(':')[0].strip('* ')
@@ -412,6 +503,13 @@ class Scene(QtGui.QGraphicsView):
         """ Called to delete nodes
         """
 
+        if len(self.__topology.selectedItems()) > 1:
+            reply = QtGui.QMessageBox.question(self, translate("Scene", "Message"), translate("Scene", "Do you really want to delete these devices?"), 
+                                            QtGui.QMessageBox.Yes, QtGui.QMessageBox.No)
+
+            if reply == QtGui.QMessageBox.No:
+                return
+
         ok_to_delete_node = True
         for item in self.__topology.selectedItems():
             if not isinstance(item, Annotation) and not isinstance(item, Pixmap) and not isinstance(item, AbstractShapeItem):
@@ -448,6 +546,19 @@ class Scene(QtGui.QGraphicsView):
             zvalue = item.zValue()
             command = undo.NewZValue(item, zvalue + 1)
             self.__topology.undoStack.push(command)
+            
+    def slotchangeSymbol(self):
+        """ Change a device's symbol
+        """
+        
+        item_list = []
+        for item in self.__topology.selectedItems():
+            if isinstance(item, AbstractNode):
+                item_list.append(item)
+        if len(item_list):
+            dialog = SymbolDialog(item_list)
+            dialog.show()
+            dialog.exec_()
 
     def slotConsole(self):
         """ Slot called to launch a console on the selected items
@@ -464,6 +575,14 @@ class Scene(QtGui.QGraphicsView):
         for item in self.__topology.selectedItems():
             if isinstance(item, IOSRouter) or isinstance(item, AnyEmuDevice):
                 item.changeConsolePort()
+                
+    def slotChangeAUXPort(self):
+        """ Slot called to change the aux port
+        """
+
+        for item in self.__topology.selectedItems():
+            if isinstance(item, IOSRouter):
+                item.changeAUXPort()
 
     def slotStartupConfig(self):
         """ Slot called to change the startup-config
@@ -484,7 +603,14 @@ class Scene(QtGui.QGraphicsView):
     def slotStopNode(self):
         """ Slot called to stop the selected items
         """
+     
+        if len(self.__topology.selectedItems()) > 1:
+            reply = QtGui.QMessageBox.question(self, translate("Scene", "Message"), translate("Scene", "Do you really want to stop these devices?"), 
+                                            QtGui.QMessageBox.Yes, QtGui.QMessageBox.No)
 
+            if reply == QtGui.QMessageBox.No:
+                return
+     
         for item in self.__topology.selectedItems():
             if isinstance(item, IOSRouter) or isinstance(item, AnyEmuDevice):
                 item.stopNode()
@@ -501,8 +627,15 @@ class Scene(QtGui.QGraphicsView):
         """ Slot called to reload the selected items
         """
 
+        if len(self.__topology.selectedItems()) > 1:
+            reply = QtGui.QMessageBox.question(self, translate("Scene", "Message"), translate("Scene", "Do you really want to reload these devices?"), 
+                                            QtGui.QMessageBox.Yes, QtGui.QMessageBox.No)
+
+            if reply == QtGui.QMessageBox.No:
+                return
+
         for item in self.__topology.selectedItems():
-            if  isinstance(item, IOSRouter):
+            if isinstance(item, IOSRouter) or isinstance(item, AnyEmuDevice):
                 item.reloadNode()
                 
     def getSourceNode(self):
@@ -586,9 +719,11 @@ class Scene(QtGui.QGraphicsView):
         self.scale(scale_factor, scale_factor)
 
     def wheelEvent(self, event):
-        """ Zoom with the mouse wheel
+        """ Zoom or scroll with the mouse wheel
         """
-        self.scaleView(pow(2.0, -event.delta() / 240.0))
+
+        if event.orientation() == QtCore.Qt.Vertical:
+            self.scaleView(pow(2.0, event.delta() / 240.0))
 
     def keyPressEvent(self, event):
         """ key press handler
@@ -672,16 +807,29 @@ class Scene(QtGui.QGraphicsView):
             item_type = type(item)
             if item_type == Ethernet or item_type == Serial:
                 show = False
+
+        # This statement checks to see if either the middle mouse is pressed
+        # or a combination of the right and left mouse buttons is pressed to start dragging the view
+        if show and event.buttons() == (QtCore.Qt.LeftButton | QtCore.Qt.RightButton) or event.buttons() & QtCore.Qt.MidButton:
+            self.lastMousePos = self.mapFromGlobal(event.globalPos())
+            self.sceneDragging = True
+            globals.GApp.scene.setCursor(QtCore.Qt.ClosedHandCursor)
+            return 
                 
         if show and event.modifiers() & QtCore.Qt.ShiftModifier and event.button() == QtCore.Qt.LeftButton and item and not globals.addingLinkFlag:
             item.setSelected(True)
         elif show and event.button() == QtCore.Qt.RightButton and not globals.addingLinkFlag:
             if item:
-                if not event.modifiers() & QtCore.Qt.ShiftModifier:
-                    for it in globals.GApp.topology.items():
-                        it.setSelected(False)
-                item.setSelected(True)
-                self.showContextualMenu()
+                #Prevent right clicking on a selected item from de-selecting all other items
+                if not item.isSelected():
+                    if not event.modifiers() & QtCore.Qt.ShiftModifier:
+                        for it in self.__topology.items():
+                            it.setSelected(False)
+                    item.setSelected(True)
+                    self.showContextualMenu()
+                else:
+                    self.showContextualMenu()
+            # When more than one item is selected display the contextual menu even if mouse is not above an item
             elif len(self.__topology.selectedItems()) > 1:
                 self.showContextualMenu()
         elif event.button() == QtCore.Qt.LeftButton and globals.addingNote:
@@ -715,18 +863,44 @@ class Scene(QtGui.QGraphicsView):
         else:
             QtGui.QGraphicsView.mousePressEvent(self, event)
 
+    def mouseReleaseEvent(self, event):
+        """ If The Left and Right mouse buttons are not still pressed TOGETHER and neither is the middle button
+            this means the user is no longer trying to drag the scene
+        """
+        
+        if self.sceneDragging and not event.buttons() == (QtCore.Qt.LeftButton | QtCore.Qt.RightButton) and not event.buttons() & QtCore.Qt.MidButton:
+            self.sceneDragging = False
+            globals.GApp.scene.setCursor(QtCore.Qt.ArrowCursor)
+        else:   
+            QtGui.QGraphicsView.mouseReleaseEvent(self, event)
+
     def mouseDoubleClickEvent(self, event):
 
         item = self.itemAt(event.pos())
         if not globals.addingLinkFlag and isinstance(item, AbstractNode):
             item.setSelected(True)
-            self.slotConfigNode()
+            if (isinstance(item, IOSRouter) or isinstance(item, AnyEmuDevice)) and item.isStarted():
+                self.slotConsole()
+            else:
+                self.slotConfigNode()
         else:
             QtGui.QGraphicsView.mouseDoubleClickEvent(self, event)
 
     def mouseMoveEvent(self, event):
-
-        if (self.newedge):
+        """ This if statement event checks to see if the user is dragging the scene
+            if so it sets the value of the scene scroll bars based on the change between
+            the previous and current mouse position
+        """
+        
+        if self.sceneDragging:
+            mappedGlobalPos = self.mapFromGlobal(event.globalPos())
+            hBar = self.horizontalScrollBar()
+            vBar = self.verticalScrollBar()
+            delta = mappedGlobalPos - self.lastMousePos;
+            hBar.setValue(hBar.value() + (delta.x() if globals.GApp.isRightToLeft() else -delta.x()))
+            vBar.setValue(vBar.value() - delta.y())
+            self.lastMousePos = mappedGlobalPos
+        elif (self.newedge):
             self.newedge.setMousePoint(self.mapToScene(event.pos()))
             event.ignore()
         else:
