@@ -19,7 +19,7 @@
 # code@gns3.net
 #
 
-import os, glob, socket, sys
+import os, glob, socket, sys, base64
 import GNS3.Dynagen.dynamips_lib as lib
 import GNS3.Dynagen.qemu_lib as qlib
 import GNS3.Globals as globals
@@ -202,9 +202,9 @@ class Topology(QtGui.QGraphicsScene):
             hypervisor_conf = globals.GApp.hypervisors[external_hypervisor_key]
             # use project workdir in priority
             if globals.GApp.workspace.projectWorkdir:
-                self.dynagen.defaults_config['workingdir'] =  globals.GApp.workspace.projectWorkdir
+                self.dynagen.defaults_config['workingdir'] = globals.GApp.workspace.projectWorkdir
             elif hypervisor_conf.workdir:
-                self.dynagen.defaults_config['workingdir'] =  hypervisor_conf.workdir
+                self.dynagen.defaults_config['workingdir'] = hypervisor_conf.workdir
             dynamips_hypervisor = self.dynagen.create_dynamips_hypervisor(host, int(port))
             if not dynamips_hypervisor:
                 QtGui.QMessageBox.critical(globals.GApp.mainWindow, translate("Topology", "Hypervisor"),
@@ -218,9 +218,28 @@ class Topology(QtGui.QGraphicsScene):
             dynamips_hypervisor.udp = hypervisor_conf.baseUDP
             dynamips_hypervisor.starting_udp = hypervisor_conf.baseUDP
             dynamips_hypervisor.baseconsole = hypervisor_conf.baseConsole
+            dynamips_hypervisor.baseaux = hypervisor_conf.baseAUX
         node.set_hypervisor(dynamips_hypervisor)
         return True
 
+    def applyIOSBaseConfig(self, node, config_path):
+        """ Apply IOS base config
+        """
+
+        try:
+            f = open(config_path, 'r')
+            config = f.read()
+            f.close()
+            config = '!\n' + config.replace('\r', "")
+            config = config.replace('%h', node.router.name)
+            encoded = ("").join(base64.encodestring(config).split())
+            node.router.config_b64 = encoded
+        except IOError, e:
+            QtGui.QMessageBox.warning(globals.GApp.mainWindow, unicode(translate("Topology", "IOS Base config")), unicode(translate("Topology", "%s: %s")) % (config_path, e[1]))
+        except:
+            debug("Cannot apply IOS base config")
+            pass
+
     def preConfigureNode(self, node, image_conf):
         """ Apply settings on node
         """
@@ -328,13 +347,14 @@ class Topology(QtGui.QGraphicsScene):
 
         command = undo.AddNode(self, node)
         self.undoStack.push(command)
-        
+
     def addNode(self, node, fromScene=False):
         """ Add node in the topology
             node: object
         """
 
         try:
+            iosConfig = None
             if isinstance(node, IOSRouter):
                 if len(globals.GApp.iosimages.keys()) == 0:
                     # no IOS images configured, users have to register an IOS
@@ -350,9 +370,12 @@ class Topology(QtGui.QGraphicsScene):
                 if len(selected_images) == 0:
                     QtGui.QMessageBox.warning(globals.GApp.mainWindow, translate("Topology", "IOS image"),
                                               unicode(translate("Topology", "No image for platform %s")) % node.platform)
+                    init_router_id(node.id)
                     return False
 
-                if len(selected_images) > 1:
+                if node.image_reference:
+                    image_to_use = node.image_reference
+                elif len(selected_images) > 1:
                     for image in selected_images:
                         conf = globals.GApp.iosimages[image]
                         if conf.default:
@@ -360,16 +383,20 @@ class Topology(QtGui.QGraphicsScene):
                             break
                     if not image_to_use:
                         (selection,  ok) = QtGui.QInputDialog.getItem(globals.GApp.mainWindow, translate("Topology", "IOS image"),
-                                                                      translate("Topology", "Please choose an image"), selected_images, 0, False)
+                                                                      translate("Topology", "Please choose an image:"), selected_images, 0, False)
                         if ok:
                             image_to_use = unicode(selection)
                         else:
+                            init_router_id(node.id)
                             return False
                 else:
                     image_to_use = selected_images[0]
 
+                node.image_reference = image_to_use
                 image_conf = globals.GApp.iosimages[image_to_use]
                 debug("Use image: " + image_to_use)
+                if image_conf.baseconfig:
+                    iosConfig = image_conf.baseconfig
                 if image_conf.default_ram:
                     debug("Set default RAM: " + str(image_conf.default_ram))
                     node.default_ram = image_conf.default_ram
@@ -377,11 +404,14 @@ class Topology(QtGui.QGraphicsScene):
                     # no hypervisor selected, allocate a new hypervisor for the node
                     if globals.GApp.systconf['dynamips'].path == '':
                         QtGui.QMessageBox.warning(globals.GApp.mainWindow, translate("Topology", "Hypervisor"), translate("Topology", "Please configure the path to Dynamips"))
+                        init_router_id(node.id)
                         return False
                     if not globals.GApp.HypervisorManager:
                         QtGui.QMessageBox.warning(globals.GApp.mainWindow, translate("Topology", "Hypervisor"), translate("Topology", "Please test the path to Dynamips in preferences"))
+                        init_router_id(node.id)
                         return False
                     if not globals.GApp.HypervisorManager.allocateHypervisor(node):
+                        init_router_id(node.id)
                         return False
                     # give a warning if the IOS path is not accessible
                     if not os.access(image_conf.filename, os.F_OK):
@@ -389,41 +419,51 @@ class Topology(QtGui.QGraphicsScene):
                 else:
                     # use an external hypervisor
                     if self.useExternalHypervisor(node, image_conf.hypervisors) == False:
+                        init_router_id(node.id)
                         return False
                 self.preConfigureNode(node, image_conf)
 
             if isinstance(node, QemuDevice):
 
                 if len(globals.GApp.qemuimages) == 0:
-                    QtGui.QMessageBox.warning(globals.GApp.mainWindow, translate("Topology", "Qemu image"), translate("Topology", "Please configure a Qemu image"))
+                    QtGui.QMessageBox.warning(globals.GApp.mainWindow, translate("Topology", "Qemu image"), translate("Topology", "Please configure a Qemu host:"))
+                    init_emu_id(node.id)
                     return False
                
-                images = []
+                devices = []
                 for name in globals.GApp.qemuimages.keys():
-                    images.append(name)
+                    devices.append(name)
                 
-                if len(globals.GApp.qemuimages) > 1:
+                if node.image_reference:
+                    conf = globals.GApp.qemuimages[node.image_reference]
+                elif len(globals.GApp.qemuimages) > 1:
 
-                    (selection,  ok) = QtGui.QInputDialog.getItem(globals.GApp.mainWindow, translate("Topology", "Qemu image"),
-                                                                      translate("Topology", "Please choose an image"), images, 0, False)
+                    (selection,  ok) = QtGui.QInputDialog.getItem(globals.GApp.mainWindow, translate("Topology", "Qemu host"),
+                                                                      translate("Topology", "Please choose a Qemu host"), devices, 0, False)
                     if ok:
-                        image_to_use = unicode(selection)
+                        device_to_use = unicode(selection)
                     else:
+                        init_emu_id(node.id)
                         return False
-                    conf = globals.GApp.qemuimages[image_to_use]
+                    conf = globals.GApp.qemuimages[device_to_use]
+                    node.image_reference = device_to_use
+
                 else:
-                    conf = globals.GApp.qemuimages[images[0]]
+                    conf = globals.GApp.qemuimages[devices[0]]
+                    node.image_reference = devices[0]
 
                 # give a warning if the Qemu image path is not accessible
                 if not os.access(conf.filename, os.F_OK) and globals.GApp.systconf['qemu'].enable_QemuManager:
                     QtGui.QMessageBox.warning(globals.GApp.mainWindow, translate("Topology", "Qemu image"), unicode(translate("Topology", "%s seems to not exist, please check")) % conf.filename)
 
                 if self.emuDeviceSetup(node) == False:
+                    init_emu_id(node.id)
                     return False
                 
                 debug("Set default image " + conf.filename + " for node type %s, model %r" % (type(node), node.model))
                 node.set_image(conf.filename, node.model)
                 node.set_int_option('ram', conf.memory)
+                node.set_int_option('nics', conf.nic_nb)
                 node.set_string_option('netcard', conf.nic)
                 node.set_string_option('kqemu', conf.kqemu)
                 node.set_string_option('kvm', conf.kvm)                
@@ -431,110 +471,201 @@ class Topology(QtGui.QGraphicsScene):
 
             if isinstance(node, JunOS):
 
-                if not globals.GApp.systconf['qemu'].default_junos_image:
-                    QtGui.QMessageBox.warning(globals.GApp.mainWindow, translate("Topology", "JunOS image"), translate("Topology", "Please configure a default JunOS image"))
+                if len(globals.GApp.junosimages) == 0:
+                    QtGui.QMessageBox.warning(globals.GApp.mainWindow, translate("Topology", "JunOS"), translate("Topology", "Please configure a JunOS"))
+                    init_emu_id(node.id)
                     return False
+               
+                devices = []
+                for name in globals.GApp.junosimages.keys():
+                    devices.append(name)
                 
+                if node.image_reference:
+                    conf = globals.GApp.junosimages[node.image_reference]
+                elif len(globals.GApp.junosimages) > 1:
+
+                    (selection,  ok) = QtGui.QInputDialog.getItem(globals.GApp.mainWindow, translate("Topology", "JunOS image"),
+                                                                      translate("Topology", "Please choose a JunOS"), devices, 0, False)
+                    if ok:
+                        device_to_use = unicode(selection)
+                    else:
+                        init_emu_id(node.id)
+                        return False
+                    conf = globals.GApp.junosimages[device_to_use]
+                    node.image_reference = device_to_use
+                else:
+                    conf = globals.GApp.junosimages[devices[0]]
+                    node.image_reference = devices[0]
+
                 # give a warning if the JunOS image path is not accessible
-                if not os.access(globals.GApp.systconf['qemu'].default_junos_image, os.F_OK) and globals.GApp.systconf['qemu'].enable_QemuManager:
-                    QtGui.QMessageBox.warning(globals.GApp.mainWindow, translate("Topology", "JunOS image"), unicode(translate("Topology", "%s seems to not exist, please check")) % globals.GApp.systconf['qemu'].default_junos_image)
+                if not os.access(conf.filename, os.F_OK) and globals.GApp.systconf['qemu'].enable_QemuManager:
+                    QtGui.QMessageBox.warning(globals.GApp.mainWindow, translate("Topology", "JunOS image"), unicode(translate("Topology", "%s seems to not exist, please check")) % conf.filename)
                 
                 if self.emuDeviceSetup(node) == False:
+                    init_emu_id(node.id)
                     return False
                 
-                debug("Set default image " + globals.GApp.systconf['qemu'].default_junos_image + " for node type %s, model %r" % (type(node), node.model))
-                node.set_image(globals.GApp.systconf['qemu'].default_junos_image, node.model)
-                node.set_int_option('ram', globals.GApp.systconf['qemu'].default_junos_memory)
-                node.set_string_option('netcard', globals.GApp.systconf['qemu'].default_junos_nic)
-                node.set_string_option('kqemu', globals.GApp.systconf['qemu'].default_junos_kqemu)
-                node.set_string_option('kvm', globals.GApp.systconf['qemu'].default_junos_kvm)                
-                node.set_string_option('options', globals.GApp.systconf['qemu'].default_junos_options)
+                debug("Set default image " + conf.filename + " for node type %s, model %r" % (type(node), node.model))
+                node.set_image(conf.filename, node.model)
+                node.set_int_option('ram', conf.memory)
+                node.set_int_option('nics', conf.nic_nb)
+                node.set_string_option('netcard', conf.nic)
+                node.set_string_option('kqemu', conf.kqemu)
+                node.set_string_option('kvm', conf.kvm)                
+                node.set_string_option('options', conf.options)
                 
             if isinstance(node, IDS):
 
-                if not globals.GApp.systconf['qemu'].default_ids_image1 or not globals.GApp.systconf['qemu'].default_ids_image2:
-                    QtGui.QMessageBox.warning(globals.GApp.mainWindow, translate("Topology", "IDS images"), translate("Topology", "Please configure the default IDS images"))
+                if len(globals.GApp.idsimages) == 0:
+                    QtGui.QMessageBox.warning(globals.GApp.mainWindow, translate("Topology", "IDS"), translate("Topology", "Please configure an IDS"))
+                    init_emu_id(node.id)
                     return False
-                
-                # give a warning if the IDS image paths are not accessible
-                if not os.access(globals.GApp.systconf['qemu'].default_ids_image1, os.F_OK) and globals.GApp.systconf['qemu'].enable_QemuManager:
-                    QtGui.QMessageBox.warning(globals.GApp.mainWindow, translate("Topology", "IDS images"), unicode(translate("Topology", "%s seems to not exist, please check")) % globals.GApp.systconf['qemu'].default_ids_image1)
+               
+                devices = []
+                for name in globals.GApp.idsimages.keys():
+                    devices.append(name)
+
+                if node.image_reference:
+                    conf = globals.GApp.idsimages[node.image_reference]
+                elif len(globals.GApp.idsimages) > 1:
+
+                    (selection,  ok) = QtGui.QInputDialog.getItem(globals.GApp.mainWindow, translate("Topology", "IDS"),
+                                                                      translate("Topology", "Please choose an IDS"), devices, 0, False)
+                    if ok:
+                        device_to_use = unicode(selection)
+                    else:
+                        init_emu_id(node.id)
+                        return False
+
+                    conf = globals.GApp.idsimages[device_to_use]
+                    node.image_reference = device_to_use
+                else:
+                    conf = globals.GApp.idsimages[devices[0]]
+                    node.image_reference = devices[0]
 
                 # give a warning if the IDS image paths are not accessible
-                if not os.access(globals.GApp.systconf['qemu'].default_ids_image2, os.F_OK) and globals.GApp.systconf['qemu'].enable_QemuManager:
-                    QtGui.QMessageBox.warning(globals.GApp.mainWindow, translate("Topology", "IDS images"), unicode(translate("Topology", "%s seems to not exist, please check")) % globals.GApp.systconf['qemu'].default_ids_image2)
+                if not os.access(conf.image1, os.F_OK) and globals.GApp.systconf['qemu'].enable_QemuManager:
+                    QtGui.QMessageBox.warning(globals.GApp.mainWindow, translate("Topology", "IDS images"), unicode(translate("Topology", "%s seems to not exist, please check")) % conf.image1)
+                if not os.access(conf.image2, os.F_OK) and globals.GApp.systconf['qemu'].enable_QemuManager:
+                    QtGui.QMessageBox.warning(globals.GApp.mainWindow, translate("Topology", "IDS images"), unicode(translate("Topology", "%s seems to not exist, please check")) % conf.image2)
           
                 if self.emuDeviceSetup(node) == False:
+                    init_emu_id(node.id)
                     return False
 
                 # No default image for IDS
                 node.set_image('None', node.model)
-                debug("Set image1 " + globals.GApp.systconf['qemu'].default_ids_image1 + " for node type %s, model %r" % (type(node), node.model))
-                debug("Set image2 " + globals.GApp.systconf['qemu'].default_ids_image2 + " for node type %s, model %r" % (type(node), node.model))
-                node.set_string_option('image1', globals.GApp.systconf['qemu'].default_ids_image1)
-                node.set_string_option('image2', globals.GApp.systconf['qemu'].default_ids_image2)
-                node.set_int_option('ram', globals.GApp.systconf['qemu'].default_ids_memory)
-                node.set_string_option('netcard', globals.GApp.systconf['qemu'].default_ids_nic)
-                node.set_string_option('kqemu', globals.GApp.systconf['qemu'].default_ids_kqemu)
-                node.set_string_option('kvm', globals.GApp.systconf['qemu'].default_ids_kvm)                
-                node.set_string_option('options', globals.GApp.systconf['qemu'].default_ids_options)
+                debug("Set image1 " + conf.image1 + " for node type %s, model %r" % (type(node), node.model))
+                debug("Set image2 " + conf.image2 + " for node type %s, model %r" % (type(node), node.model))
+                node.set_string_option('image1', conf.image1)
+                node.set_string_option('image2', conf.image2)
+                node.set_int_option('ram', conf.memory)
+                node.set_int_option('nics', conf.nic_nb)
+                node.set_string_option('netcard', conf.nic)
+                node.set_string_option('kqemu', conf.kqemu)
+                node.set_string_option('kvm', conf.kvm)                
+                node.set_string_option('options', conf.options)
 
             if isinstance(node, ASA):
-
-                if not globals.GApp.systconf['qemu'].default_asa_kernel:
-                    QtGui.QMessageBox.warning(globals.GApp.mainWindow, translate("Topology", "ASA kernel"), translate("Topology", "Please configure a default ASA kernel"))
-                    return False
                 
-                if not globals.GApp.systconf['qemu'].default_asa_initrd:
-                    QtGui.QMessageBox.warning(globals.GApp.mainWindow, translate("Topology", "ASA initrd"), translate("Topology", "Please configure a default ASA initrd"))
+                if len(globals.GApp.asaimages) == 0:
+                    QtGui.QMessageBox.warning(globals.GApp.mainWindow, translate("Topology", "ASA"), translate("Topology", "Please configure an ASA"))
+                    init_emu_id(node.id)
                     return False
-                
+               
+                devices = []
+                for name in globals.GApp.asaimages.keys():
+                    devices.append(name)
 
+                if node.image_reference:
+                    conf = globals.GApp.asaimages[node.image_reference]
+                elif len(globals.GApp.asaimages) > 1:
+
+                    (selection,  ok) = QtGui.QInputDialog.getItem(globals.GApp.mainWindow, translate("Topology", "ASA"),
+                                                                      translate("Topology", "Please choose an ASA"), devices, 0, False)
+                    if ok:
+                        device_to_use = unicode(selection)
+                    else:
+                        init_emu_id(node.id)
+                        return False
+                    conf = globals.GApp.asaimages[device_to_use]
+                    node.image_reference = device_to_use
+                else:
+                    conf = globals.GApp.asaimages[devices[0]]
+                    node.image_reference = devices[0]
+                
                 # give a warning if the ASA initrd path is not accessible
-                if not os.access(globals.GApp.systconf['qemu'].default_asa_initrd, os.F_OK) and globals.GApp.systconf['qemu'].enable_QemuManager:
-                    QtGui.QMessageBox.warning(globals.GApp.mainWindow, translate("Topology", "ASA initrd"), unicode(translate("Topology", "%s seems to not exist, please check")) % globals.GApp.systconf['qemu'].default_asa_initrd)
+                if not os.access(conf.initrd, os.F_OK) and globals.GApp.systconf['qemu'].enable_QemuManager:
+                    QtGui.QMessageBox.warning(globals.GApp.mainWindow, translate("Topology", "ASA initrd"), unicode(translate("Topology", "%s seems to not exist, please check")) % conf.initrd)
              
                 # give a warning if the ASA kernel path is not accessible
-                if not os.access(globals.GApp.systconf['qemu'].default_asa_kernel, os.F_OK) and globals.GApp.systconf['qemu'].enable_QemuManager:
-                    QtGui.QMessageBox.warning(globals.GApp.mainWindow, translate("Topology", "ASA kernel"), unicode(translate("Topology", "%s seems to not exist, please check")) % globals.GApp.systconf['qemu'].default_asa_kernel)
+                if not os.access(conf.kernel, os.F_OK) and globals.GApp.systconf['qemu'].enable_QemuManager:
+                    QtGui.QMessageBox.warning(globals.GApp.mainWindow, translate("Topology", "ASA kernel"), unicode(translate("Topology", "%s seems to not exist, please check")) % conf.kernel)
    
                 if self.emuDeviceSetup(node) == False:
+                    init_emu_id(node.id)
                     return False
                 
-                debug("Set default initrd " + globals.GApp.systconf['qemu'].default_asa_initrd + " for node type %s, model %r" % (type(node), node.model))
-                debug("Set default kernel " + globals.GApp.systconf['qemu'].default_asa_kernel + " for node type %s, model %r" % (type(node), node.model))
+                debug("Set default initrd " + conf.initrd + " for node type %s, model %r" % (type(node), node.model))
+                debug("Set default kernel " + conf.kernel + " for node type %s, model %r" % (type(node), node.model))
                 
                 # No image for ASA
                 node.set_image('None', node.model)
-                node.set_int_option('ram', globals.GApp.systconf['qemu'].default_asa_memory)
-                node.set_string_option('netcard', globals.GApp.systconf['qemu'].default_asa_nic)
-                node.set_string_option('kqemu', globals.GApp.systconf['qemu'].default_asa_kqemu)
-                node.set_string_option('kvm', globals.GApp.systconf['qemu'].default_asa_kvm)     
-                node.set_string_option('initrd', globals.GApp.systconf['qemu'].default_asa_initrd)
-                node.set_string_option('kernel', globals.GApp.systconf['qemu'].default_asa_kernel)
-                node.set_string_option('kernel_cmdline', globals.GApp.systconf['qemu'].default_asa_kernel_cmdline)
-                node.set_string_option('options', globals.GApp.systconf['qemu'].default_asa_options)
+                node.set_int_option('ram', conf.memory)
+                node.set_int_option('nics', conf.nic_nb)
+                node.set_string_option('netcard', conf.nic)
+                node.set_string_option('kqemu', conf.kqemu)
+                node.set_string_option('kvm', conf.kvm)
+                node.set_string_option('initrd', conf.initrd)
+                node.set_string_option('kernel', conf.kernel)
+                node.set_string_option('kernel_cmdline', conf.kernel_cmdline)
+                node.set_string_option('options', conf.options)
 
             if isinstance(node, FW):
                 
-                if not globals.GApp.systconf['qemu'].default_pix_image:
-                    QtGui.QMessageBox.warning(globals.GApp.mainWindow, translate("Topology", "PIX image"), translate("Topology", "Please configure a default PIX image"))
+                if len(globals.GApp.piximages) == 0:
+                    QtGui.QMessageBox.warning(globals.GApp.mainWindow, translate("Topology", "PIX"), translate("Topology", "Please configure a PIX"))
+                    init_emu_id(node.id)
                     return False
+               
+                devices = []
+                for name in globals.GApp.piximages.keys():
+                    devices.append(name)
+
+                if node.image_reference:
+                    conf = globals.GApp.piximages[node.image_reference]
+                elif len(globals.GApp.piximages) > 1:
+
+                    (selection,  ok) = QtGui.QInputDialog.getItem(globals.GApp.mainWindow, translate("Topology", "PIX"),
+                                                                      translate("Topology", "Please choose a PIX"), devices, 0, False)
+                    if ok:
+                        device_to_use = unicode(selection)
+                    else:
+                        init_emu_id(node.id)
+                        return False
+                    conf = globals.GApp.piximages[device_to_use]
+                    node.image_reference = device_to_use
+                else:
+                    conf = globals.GApp.piximages[devices[0]]
+                    node.image_reference = devices[0]
                     
                 # give a warning if the PIX image path is not accessible
-                    if not os.access(globals.GApp.systconf['qemu'].default_pix_image, os.F_OK) and globals.GApp.systconf['qemu'].enable_QemuManager:
-                        QtGui.QMessageBox.warning(globals.GApp.mainWindow, translate("Topology", "PIX image"), unicode(translate("Topology", "%s seems to not exist, please check")) % globals.GApp.systconf['qemu'].default_pix_image)
+                if not os.access(conf.filename, os.F_OK) and globals.GApp.systconf['qemu'].enable_QemuManager:
+                    QtGui.QMessageBox.warning(globals.GApp.mainWindow, translate("Topology", "PIX image"), unicode(translate("Topology", "%s seems to not exist, please check")) % conf.filename)
+                
                 if self.emuDeviceSetup(node) == False:
+                    init_emu_id(node.id)
                     return False
                 
-                debug("Set default image " + globals.GApp.systconf['qemu'].default_pix_image + " for node type %s, model %r" % (type(node), node.model))
-                node.set_image(globals.GApp.systconf['qemu'].default_pix_image, node.model)
-                node.set_int_option('ram', globals.GApp.systconf['qemu'].default_pix_memory)
-                node.set_string_option('netcard', globals.GApp.systconf['qemu'].default_pix_nic)
-                node.set_string_option('key', globals.GApp.systconf['qemu'].default_pix_key)
-                node.set_string_option('serial', globals.GApp.systconf['qemu'].default_pix_serial)
-                node.set_string_option('kqemu', globals.GApp.systconf['qemu'].default_pix_kqemu)
-                node.set_string_option('options', globals.GApp.systconf['qemu'].default_pix_options)
+                debug("Set default image " + conf.filename + " for node type %s, model %r" % (type(node), node.model))
+                node.set_image(conf.filename, node.model)
+                node.set_int_option('ram', conf.memory)
+                node.set_int_option('nics', conf.nic_nb)
+                node.set_string_option('netcard', conf.nic)
+                node.set_string_option('key', conf.key)
+                node.set_string_option('serial', conf.serial)
+                node.set_string_option('kqemu', conf.kqemu)
+                node.set_string_option('options', conf.options)
 
             QtCore.QObject.connect(node, QtCore.SIGNAL("Add link"), globals.GApp.scene.slotAddLink)
             QtCore.QObject.connect(node, QtCore.SIGNAL("Delete link"), globals.GApp.scene.slotDeleteLink)
@@ -544,6 +675,13 @@ class Topology(QtGui.QGraphicsScene):
 
             if node.configNode() == False:
                 self.deleteNode(node.id)
+                if isinstance(node, AnyEmuDevice):
+                    init_emu_id(node.id)
+                if isinstance(node, IOSRouter):
+                    init_router_id(node.id)
+                
+            if iosConfig:
+                self.applyIOSBaseConfig(node, iosConfig)
 
         except (lib.DynamipsVerError, lib.DynamipsError), msg:
             if isinstance(node, IOSRouter):
@@ -555,10 +693,18 @@ class Topology(QtGui.QGraphicsScene):
                             print "Warning: " + file + " is not writable because of different rights, please delete this file manually if dynamips was not able to create this router"
             QtGui.QMessageBox.critical(globals.GApp.mainWindow, translate("Topology", "Dynamips error"),  unicode(msg))
             self.deleteNode(node.id)
+            if isinstance(node, AnyEmuDevice):
+                init_emu_id(node.id)
+            if isinstance(node, IOSRouter):
+                init_router_id(node.id)
             return False
         except (lib.DynamipsErrorHandled, socket.error):
             QtGui.QMessageBox.critical(globals.GApp.mainWindow, translate("Topology", "Dynamips error"), translate("Topology", "Connection lost"))
             self.deleteNode(node.id)
+            if isinstance(node, AnyEmuDevice):
+                init_emu_id(node.id)
+            if isinstance(node, IOSRouter):
+                init_router_id(node.id)
             return False
 
         globals.GApp.mainWindow.treeWidget_TopologySummary.refresh()
@@ -623,6 +769,7 @@ class Topology(QtGui.QGraphicsScene):
         self.removeItem(node)
         del self.__nodes[id]
         globals.GApp.mainWindow.treeWidget_TopologySummary.refresh()
+        globals.GApp.mainWindow.capturesDock.refresh()
         # Work-around QGraphicsSvgItem caching bug:
         # Forcing to clear the QPixmapCache on node delete.
         # FIXME: in Qt 4.4
@@ -665,8 +812,8 @@ class Topology(QtGui.QGraphicsScene):
             multi = 0
         
         if (globals.currentLinkType == globals.Enum.LinkType.Serial or globals.currentLinkType == globals.Enum.LinkType.ATM) or \
-            (globals.currentLinkType == globals.Enum.LinkType.Manual and ((srcif[0] == 's' or srcif[0] == 'a' or dstif[0] == 's' or dstif[0] == 'a') or \
-            (isinstance(src_node, ATMSW) or isinstance(src_node, FRSW) or isinstance(dest_node, ATMSW) or isinstance(dest_node, FRSW)))):
+            (globals.currentLinkType == globals.Enum.LinkType.Manual and (((srcif[0] == 's' or srcif[0] == 'a') or (dstif[0] == 's' or dstif[0] == 'a')) or \
+            ((isinstance(src_node, ATMSW) or isinstance(src_node, FRSW)) or (isinstance(dest_node, ATMSW) or isinstance(dest_node, FRSW))))):
             # interface is serial or ATM
             link = Serial(self.__nodes[srcid], srcif, self.__nodes[dstid], dstif, Multi=multi)
         else:
@@ -727,7 +874,8 @@ class Topology(QtGui.QGraphicsScene):
         elif not isinstance(src_node, IOSRouter) and not isinstance(dst_node, IOSRouter):
 
             if ((isinstance(src_node, AnyEmuDevice) or isinstance(src_node, Cloud)) and type(dst_node) in (ATMSW, FRSW, ATMBR)) \
-                or ((isinstance(dst_node, AnyEmuDevice) or isinstance(dst_node, Cloud)) and type(src_node) in (ATMSW, FRSW, ATMBR)):
+                or ((isinstance(dst_node, AnyEmuDevice) or isinstance(dst_node, Cloud)) and type(src_node) in (ATMSW, FRSW, ATMBR)) \
+                or (isinstance(src_node, Cloud) and isinstance(dst_node, Cloud)):
                 QtGui.QMessageBox.critical(globals.GApp.mainWindow, translate("Topology", "Connection"),  translate("Topology", "Can't connect these devices"))
                 return False
 
@@ -882,10 +1030,14 @@ class Topology(QtGui.QGraphicsScene):
         if link in self.__links:
             self.__links.remove(link)
             if link.labelSouceIf != None:
-                globals.interfaceLabels[link.source.hostname + ' ' + link.srcIf] = link.labelSouceIf
+                if globals.interfaceLabels.has_key(link.source.hostname + ' ' + link.srcIf):
+                    del globals.interfaceLabels[link.source.hostname + ' ' + link.srcIf]
+                #globals.interfaceLabels[link.source.hostname + ' ' + link.srcIf] = link.labelSouceIf  
                 self.removeItem(link.labelSouceIf)
             if link.labelDestIf != None:
-                globals.interfaceLabels[link.dest.hostname + ' ' + link.destIf] = link.labelDestIf
+                if globals.interfaceLabels.has_key(link.dest.hostname + ' ' + link.destIf):
+                    del globals.interfaceLabels[link.dest.hostname + ' ' + link.destIf]
+                #globals.interfaceLabels[link.dest.hostname + ' ' + link.destIf] = link.labelDestIf
                 self.removeItem(link.labelDestIf)
             self.removeItem(link)
         globals.GApp.mainWindow.treeWidget_TopologySummary.refresh()
