@@ -1,7 +1,7 @@
 # -*- coding: utf-8 -*-
 # vim: expandtab ts=4 sw=4 sts=4:
 #
-# Copyright (C) 2007 GNS-3 Dev Team
+# Copyright (C) 2007-2010 GNS3 Development Team (http://www.gns3.net/team).
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License version 2 as
@@ -16,12 +16,12 @@
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 #
-# Contact: contact@gns3.net
+# code@gns3.net
 #
 
-import re, sys, string
-import subprocess as sub
+import re, sys
 import GNS3.Globals as globals
+from GNS3.Utils import translate, getWindowsInterfaces
 from PyQt4 import QtCore, QtGui, QtNetwork
 from Form_CloudPage import Ui_CloudPage
 
@@ -66,7 +66,7 @@ class Page_Cloud(QtGui.QWidget, Ui_CloudPage):
         
         self.nios = []
         if sys.platform.startswith('win'):
-            interfaces = self.getWindowsInterfaces()
+            interfaces = getWindowsInterfaces()
         else:
             interfaces = map(lambda interface: interface.name(), QtNetwork.QNetworkInterface.allInterfaces())
             self.comboBoxLinuxEth.addItems(interfaces)
@@ -74,40 +74,6 @@ class Page_Cloud(QtGui.QWidget, Ui_CloudPage):
         self.comboBoxGenEth.setSizeAdjustPolicy(QtGui.QComboBox.AdjustToContents)
         self.comboBoxGenEth.addItems(interfaces)
 
-    def getWindowsInterfaces(self):
-        """ Try to detect all available interfaces on Windows
-        """
-
-        try:
-            import _winreg
-        except:
-            pass
-        
-        interfaces = []
-        dynamips = globals.GApp.systconf['dynamips']
-        if dynamips == '':
-            return []
-        try:
-            p = sub.Popen(dynamips.path + ' -e', stdout=sub.PIPE, stderr=sub.STDOUT)
-            outputlines = p.stdout.readlines()
-            p.wait()
-            for line in outputlines:
-                match = re.search(r"""^rpcap://\\Device\\NPF_({.*}).*""",  line.strip())
-                if match:
-                    interface_name = ': '
-                    try:
-                        reg_key = "SYSTEM\\CurrentControlSet\\Control\\Network\\{4D36E972-E325-11CE-BFC1-08002BE10318}\\%s\\Connection" % match.group(1)
-                        key = _winreg.OpenKey(_winreg.HKEY_LOCAL_MACHINE, reg_key, _winreg.KEY_READ)
-                        (value, typevalue) = _winreg.QueryValueEx(key, 'Name')
-                        _winreg.CloseKey(key)
-                        interface_name += value
-                    except:
-                        interface_name += "unknown name"
-                        pass
-                    interfaces.append(match.group(0) + interface_name)
-        except:
-            return []
-        return interfaces
 
     def slotSelectedGenEth(self,  index):
         """ Load the selected generic interface in lineEdit
@@ -138,7 +104,7 @@ class Page_Cloud(QtGui.QWidget, Ui_CloudPage):
         interface = unicode(self.lineEditGenEth.text())
         if interface:
             if sys.platform.startswith('win'):
-                match = re.search(r"""^rpcap://(\\Device\\NPF_{.*}).*""", interface)
+                match = re.search(r"""^rpcap://(\\Device\\NPF_{[a-fA-F0-9\-]*}).*""", interface)
                 interface = match.group(1)
             nio = 'nio_gen_eth:' + interface.lower()
             if not nio in self.nios:
@@ -152,6 +118,10 @@ class Page_Cloud(QtGui.QWidget, Ui_CloudPage):
         item = self.listWidgetGenericEth.currentItem()
         if (item != None):
             nio = unicode(item.text())
+            connected_ports = self.node.getConnectedInterfaceList()
+            if nio in connected_ports:
+                QtGui.QMessageBox.critical(globals.nodeConfiguratorWindow, 'NIOs', unicode(translate("Page_Cloud", "A link is connected with NIO %s")) % nio)
+                return
             self.nios.remove(nio)
             self.listWidgetGenericEth.takeItem(self.listWidgetGenericEth.currentRow())
             
@@ -182,6 +152,10 @@ class Page_Cloud(QtGui.QWidget, Ui_CloudPage):
         item = self.listWidgetLinuxEth.currentItem()
         if (item != None):
             nio = unicode(item.text())
+            connected_ports = self.node.getConnectedInterfaceList()
+            if nio in connected_ports:
+                QtGui.QMessageBox.critical(globals.nodeConfiguratorWindow, 'NIOs', unicode(translate("Page_Cloud", "A link is connected with NIO %s")) % nio)
+                return
             self.nios.remove(nio)
             self.listWidgetLinuxEth.takeItem(self.listWidgetLinuxEth.currentRow())
         
@@ -207,6 +181,10 @@ class Page_Cloud(QtGui.QWidget, Ui_CloudPage):
         item = self.listWidgetUDP.currentItem()
         if (item != None):
             nio = unicode(item.text())
+            connected_ports = self.node.getConnectedInterfaceList()
+            if nio in connected_ports:
+                QtGui.QMessageBox.critical(globals.nodeConfiguratorWindow, 'NIOs', unicode(translate("Page_Cloud", "A link is connected with NIO %s")) % nio)
+                return
             self.nios.remove(nio)
             self.listWidgetUDP.takeItem(self.listWidgetUDP.currentRow())
 
@@ -251,6 +229,10 @@ class Page_Cloud(QtGui.QWidget, Ui_CloudPage):
         item = self.listWidgetTAP.currentItem()
         if (item != None):
             nio = unicode(item.text())
+            connected_ports = self.node.getConnectedInterfaceList()
+            if nio in connected_ports:
+                QtGui.QMessageBox.critical(globals.nodeConfiguratorWindow, 'NIOs', unicode(translate("Page_Cloud", "A link is connected with NIO %s")) % nio)
+                return
             self.nios.remove(nio)
             self.listWidgetTAP.takeItem(self.listWidgetTAP.currentRow())
 
@@ -283,6 +265,10 @@ class Page_Cloud(QtGui.QWidget, Ui_CloudPage):
         item = self.listWidgetUNIX.currentItem()
         if (item != None):
             nio = unicode(item.text())
+            connected_ports = self.node.getConnectedInterfaceList()
+            if nio in connected_ports:
+                QtGui.QMessageBox.critical(globals.nodeConfiguratorWindow, 'NIOs', unicode(translate("Page_Cloud", "A link is connected with NIO %s")) % nio)
+                return
             self.nios.remove(nio)
             self.listWidgetUNIX.takeItem(self.listWidgetUNIX.currentRow())
 
@@ -327,6 +313,10 @@ class Page_Cloud(QtGui.QWidget, Ui_CloudPage):
         item = self.listWidgetVDE.currentItem()
         if (item != None):
             nio = unicode(item.text())
+            connected_ports = self.node.getConnectedInterfaceList()
+            if nio in connected_ports:
+                QtGui.QMessageBox.critical(globals.nodeConfiguratorWindow, 'NIOs', unicode(translate("Page_Cloud", "A link is connected with NIO %s")) % nio)
+                return
             self.nios.remove(nio)
             self.listWidgetVDE.takeItem(self.listWidgetVDE.currentRow())
 
@@ -369,7 +359,11 @@ class Page_Cloud(QtGui.QWidget, Ui_CloudPage):
         
         item = self.listWidgetNull.currentItem()
         if (item != None):
-            nio = unicode(item.text())
+            nio = unicode(item.text())  
+            connected_ports = self.node.getConnectedInterfaceList()
+            if nio in connected_ports:
+                QtGui.QMessageBox.critical(globals.nodeConfiguratorWindow, 'NIOs', unicode(translate("Page_Cloud", "A link is connected with NIO %s")) % nio)
+                return
             self.nios.remove(nio)
             self.listWidgetNull.takeItem(self.listWidgetNull.currentRow())
 
@@ -398,11 +392,11 @@ class Page_Cloud(QtGui.QWidget, Ui_CloudPage):
         """ Load the config
         """
 
-        node = globals.GApp.topology.getNode(id)
+        self.node = globals.GApp.topology.getNode(id)
         if config:
             Cloudconfig = config
         else:
-            Cloudconfig  = node.config
+            Cloudconfig = self.node.config
 
         self.nios = []
         self.listWidgetGenericEth.clear()
@@ -434,13 +428,13 @@ class Page_Cloud(QtGui.QWidget, Ui_CloudPage):
         """ Save the config
         """
 
-        node = globals.GApp.topology.getNode(id)
+        self.node = globals.GApp.topology.getNode(id)
         if config:
             Cloudconfig = config
         else:
-            Cloudconfig  = node.config
+            Cloudconfig  = self.node.duplicate_config()
 
-        Cloudconfig['nios']  = self.nios
+        Cloudconfig['nios'] = self.nios
 
         return Cloudconfig
             
