@@ -1,7 +1,7 @@
 # -*- coding: utf-8 -*-
 # vim: expandtab ts=4 sw=4 sts=4:
 #
-# Copyright (C) 2007 GNS-3 Dev Team
+# Copyright (C) 2007-2010 GNS3 Development Team (http://www.gns3.net/team).
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License version 2 as
@@ -16,12 +16,12 @@
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 #
-# Contact: contact@gns3.net
+# code@gns3.net
 #
 
 import os, re
 import GNS3.Globals as globals
-from PyQt4 import QtCore,  QtGui
+from PyQt4 import QtGui
 from Form_IOSRouterPage import Ui_IOSRouterPage
 from GNS3.Utils import translate
 import GNS3.Dynagen.dynamips_lib as lib
@@ -37,18 +37,71 @@ class Page_IOSRouter(QtGui.QWidget, Ui_IOSRouterPage):
         self.setObjectName("IOSRouter")
         self.currentNodeID = None
         self.NM_16ESW_warning = True
+        self.NPEG2_warning = True
 
         self.widget_slots = {0: self.comboBoxSlot0,
-                                        1: self.comboBoxSlot1,
-                                        2: self.comboBoxSlot2,
-                                        3: self.comboBoxSlot3,
-                                        4: self.comboBoxSlot4,
-                                        5: self.comboBoxSlot5,
-                                        6: self.comboBoxSlot6}
+                             1: self.comboBoxSlot1,
+                             2: self.comboBoxSlot2,
+                             3: self.comboBoxSlot3,
+                             4: self.comboBoxSlot4,
+                             5: self.comboBoxSlot5,
+                             6: self.comboBoxSlot6,
+                             7: self.comboBoxSlot7}
                                         
         self.widget_wics = {0: self.comboBoxWIC0,
-                                        1: self.comboBoxWIC1,
-                                        2: self.comboBoxWIC2}
+                            1: self.comboBoxWIC1,
+                            2: self.comboBoxWIC2}
+    
+    def loadSlotConfig(self, platform, chassis, router_config, node):
+        """ Load slot config
+        """
+
+        for widget in self.widget_slots.values():
+            widget.setEnabled(False)
+            widget.clear()
+
+        
+        for (slot_number, slot_modules) in lib.ADAPTER_MATRIX[platform][chassis].iteritems():
+            self.widget_slots[slot_number].setEnabled(True)
+            if type(slot_modules) == str:
+                self.widget_slots[slot_number].addItem(slot_modules)
+            elif platform == 'c7200' and slot_number == 0 and router_config['slots'][slot_number] != None:
+                self.widget_slots[slot_number].addItem(router_config['slots'][slot_number])
+            else:
+                module_list = list(slot_modules)
+                if platform == 'c7200' and slot_number == 0 and node.router.dynamips.intversion < 208.3:
+                    module_list.remove('C7200-JC-PA')
+                self.widget_slots[slot_number].addItems([''] + module_list)
+
+            if router_config['slots'][slot_number]:
+                index = self.widget_slots[slot_number].findText(router_config['slots'][slot_number])
+                if (index != -1):
+                    self.widget_slots[slot_number].setCurrentIndex(index)
+
+        if platform == 'c7200' and (router_config['npe'] != 'npe-g2' or router_config['slots'][0] != 'C7200-JC-PA'):
+            self.widget_slots[7].setEnabled(False)
+
+        for widget in self.widget_wics.values():
+            widget.setEnabled(False)
+            widget.clear()
+
+        if router_config['wics']:
+            wic_number = 0
+            for wic_name in router_config['wics']:
+                self.widget_wics[wic_number].setEnabled(True)
+                if wic_name:
+                    self.widget_wics[wic_number].addItem(wic_name)
+                else:
+                    available_wics = ['', 'WIC-1T', 'WIC-2T']
+                    if platform == 'c1700':
+                        # Ethernet WIC only available on platform c1700
+                        available_wics.append('WIC-1ENET')
+                    self.widget_wics[wic_number].addItems(available_wics)
+                    if wic_name:
+                        index = self.widget_wics[wic_number].findText(wic_name)
+                        if (index != -1):
+                            self.widget_wics[wic_number].setCurrentIndex(index)
+                wic_number += 1
 
     def loadConfig(self, id, config = None):
         """ Load the config
@@ -66,49 +119,16 @@ class Page_IOSRouter(QtGui.QWidget, Ui_IOSRouterPage):
         chassis = node.get_chassis()
         self.textLabel_Platform.setText(platform)
         self.textLabel_Model.setText(router.model_string)
-        self.textLabel_ImageIOS.setText(router_config['image'])
+        self.textLabel_ImageIOS.setText(os.path.basename(router_config['image']))
 
-        for widget in self.widget_slots.values():
-            widget.clear()
-        
-        for (slot_number, slot_modules) in lib.ADAPTER_MATRIX[platform][chassis].iteritems():
-            if type(slot_modules) == str:
-                self.widget_slots[slot_number].addItem(slot_modules)
-            elif platform == 'c7200' and slot_number == 0:
-                self.widget_slots[slot_number].addItems(list(slot_modules))
-            else:
-                self.widget_slots[slot_number].addItems([''] + list(slot_modules))
-            if router_config['slots'][slot_number]:
-                index = self.widget_slots[slot_number].findText(router_config['slots'][slot_number])
-                if (index != -1):
-                    self.widget_slots[slot_number].setCurrentIndex(index)
-
-        for widget in self.widget_wics.values():
-            widget.clear()
-
-        if router_config['wics']:
-            wic_number = 0
-            for wic_name in router_config['wics']:
-                if wic_name:
-                    self.widget_wics[wic_number].addItem(wic_name)
-                else:
-                    available_wics = ['', 'WIC-1T', 'WIC-2T']
-                    if platform == 'c1700':
-                        # Ethernet WIC only available on platform c1700
-                        available_wics.append('WIC-1ENET')
-                    self.widget_wics[wic_number].addItems(available_wics)
-                    if wic_name:
-                        index = self.widget_wics[wic_number].findText(wic_name)
-                        if (index != -1):
-                            self.widget_wics[wic_number].setCurrentIndex(index)
-                wic_number += 1
+        self.loadSlotConfig(platform, chassis, router_config, node)
         
         if platform == 'c7200':
             self.comboBoxMidplane.clear()
             self.comboBoxMidplane.addItems(['std', 'vxr'])
             self.comboBoxMidplane.setEnabled(True)
             self.comboBoxNPE.clear()
-            self.comboBoxNPE.addItems(['npe-100', 'npe-150', 'npe-175', 'npe-200', 'npe-225', 'npe-300', 'npe-400'])
+            self.comboBoxNPE.addItems(['npe-100', 'npe-150', 'npe-175', 'npe-200', 'npe-225', 'npe-300', 'npe-400', 'npe-g2'])
             self.comboBoxNPE.setEnabled(True)
         else:
             self.comboBoxMidplane.setEnabled(False)
@@ -155,7 +175,7 @@ class Page_IOSRouter(QtGui.QWidget, Ui_IOSRouterPage):
         if config:
             router_config = config
         else:
-            router_config = node.get_config()
+            router_config = node.duplicate_config()
 
         mac = str(self.lineEditMAC.text())
         if mac and not re.search(r"""^([0-9a-fA-F]{2}[:-]){5}[0-9a-fA-F]{2}$""", mac):
@@ -175,11 +195,18 @@ class Page_IOSRouter(QtGui.QWidget, Ui_IOSRouterPage):
             router_config['exec_area'] = None
 
         platform = node.get_platform()
+        chassis = node.get_chassis()
+
         if platform == 'c7200':
             if str(self.comboBoxMidplane.currentText()):
                 router_config['midplane'] = str(self.comboBoxMidplane.currentText())
             if str(self.comboBoxNPE.currentText()):
                 router_config['npe'] = str(self.comboBoxNPE.currentText())
+                if router_config['npe'] == 'npe-g2' and self.NPEG2_warning:
+                    
+                    globals.GApp.mainWindow.errorMessage.showMessage(translate("Page_IOSRouter", \
+                                                                               "Using npe-g2: there are potential bugs and your IOS image should be unpacked.\nC7200-IO-2FE, C7200-IO-GE-E, PA-2FE-TX and PA-GE are unlikely to work."))
+                    self.NPEG2_warning = False
 
         iomem = self.spinBoxIomem.value()
         if platform == 'c3600' and iomem != 5:
@@ -194,8 +221,8 @@ class Page_IOSRouter(QtGui.QWidget, Ui_IOSRouterPage):
             if module:
                 # Give an information (only once): users must use manual connections with NM-16ESW
                 if module == 'NM-16ESW' and self.NM_16ESW_warning:
+                    globals.GApp.mainWindow.errorMessage.showMessage(translate("Page_IOSRouter", "You must use 'manual mode' to connect a link with a NM-16ESW module"))
                     self.NM_16ESW_warning = False
-                    QtGui.QMessageBox.information(globals.nodeConfiguratorWindow, 'NM-16ESW', translate("Page_IOSRouter", "You must use manual mode when adding a link in NM-16ESW modules"))
                 collision = False
                 if router.slot[slot_number] and router_config['slots'][slot_number] != module:
                     interfaces = router.slot[slot_number].interfaces
@@ -230,6 +257,13 @@ class Page_IOSRouter(QtGui.QWidget, Ui_IOSRouterPage):
                 wic_name = str(widget.currentText())
                 if wic_name:
                     router_config['wics'][wic_number] = wic_name
+
+        if  router_config['slots'][0] == 'C7200-JC-PA' and router_config['npe'] != 'npe-g2':
+            QtGui.QMessageBox.critical(globals.nodeConfiguratorWindow, 'Slots', translate("Page_IOSRouter", "C7200-JC-PA can only be used with NPE-G2"))
+            router_config['slots'][0] = None
+
+        self.loadSlotConfig(platform, chassis, router_config, node)
+
         return router_config
 
 def create(dlg):
